@@ -16,8 +16,7 @@
  * Authored by: Pawel Stolowski <pawel.stolowski@canonical.com>
  */
 
-#ifndef UNITY_SCOPES_ACTIVATIONQUERYBASE_H
-#define UNITY_SCOPES_ACTIVATIONQUERYBASE_H
+#pragma once
 
 #include <unity/scopes/QueryBase.h>
 #include <unity/util/DefinesPtrs.h>
@@ -125,5 +124,3 @@ private:
 } // namespace scopes
 
 } // namespace unity
-
-#endif
