@@ -16,8 +16,7 @@
  * Authored by: Michi Henning <michi.henning@canonical.com>
  */
 
-#ifndef UNITY_SCOPES_QUERYBASE_H
-#define UNITY_SCOPES_QUERYBASE_H
+#pragma once
 
 #include <unity/scopes/QueryCtrlProxyFwd.h>
 #include <unity/scopes/ReplyProxyFwd.h>
@@ -124,12 +123,10 @@ protected:
 private:
     friend class internal::QueryObject;                    // So QueryObject can call cancel()
     friend class internal::smartscopes::SSQueryObject;     // So SSQueryObject can call cancel()
-    friend class internal::ScopeObject;                    // So ScopeObject can call set_metadata() and set_department_id()
-    friend class internal::smartscopes::SSScopeObject;     // So SSQueryObject can call set_department_id()
+    friend class internal::ScopeObject;                    // So ScopeObject can call set_department_id()
+    friend class internal::smartscopes::SSScopeObject;     // So SSScopeObject can call set_department_id()
 };
 
 } // namespace scopes
 
 } // namespace unity
-
-#endif
