@@ -16,8 +16,7 @@
  * Authored by: Michi Henning <michi.henning@canonical.com>
  */
 
-#ifndef UNITY_SCOPES_REPLY_H
-#define UNITY_SCOPES_REPLY_H
+#pragma once
 
 #include <unity/scopes/Object.h>
 #include <unity/scopes/OperationInfo.h>
@@ -93,5 +92,3 @@ protected:
 } // namespace scopes
 
 } // namespace unity
-
-#endif
