@@ -16,8 +16,7 @@
  * Authored by: Pawel Stolowski <pawel.stolowski@canonical.com>
  */
 
-#ifndef UNITY_SCOPES_RESULT_H
-#define UNITY_SCOPES_RESULT_H
+#pragma once
 
 #include <unity/scopes/Variant.h>
 #include <unity/util/DefinesPtrs.h>
@@ -230,11 +229,10 @@ public:
     */
     VariantMap serialize() const;
 
-protected:
     /// @cond
+protected:
     explicit Result(const VariantMap &variant_map);
     explicit Result(internal::ResultImpl* impl);
-    /// @endcond
 
 private:
     std::unique_ptr<internal::ResultImpl> p;
@@ -242,10 +240,9 @@ private:
     friend class internal::ResultImpl;
     friend class internal::ScopeImpl;
     friend class CategorisedResult;
+    /// @endcond
 };
 
 } // namespace scopes
 
 } // namespace unity
-
-#endif
