@@ -16,8 +16,7 @@
  * Authored by: Michi Henning <michi.henning@canonical.com>
  */
 
-#ifndef UNITY_SCOPES_RUNTIME_H
-#define UNITY_SCOPES_RUNTIME_H
+#pragma once
 
 #include <unity/scopes/ObjectProxyFwd.h>
 #include <unity/scopes/RegistryProxyFwd.h>
@@ -113,20 +112,11 @@ public:
     \brief Run a scope without going through the scope runner.
 
     This method is intended to run a scope that can not be loaded via the scope runner, such as
-    those written in languages that can not be dynamically loaded.
+    those written in languages that cannot be dynamically loaded.
 
     \param scope_base The scope implementation
-    \param runtime_ini_file The full path of runtime configuration file. Passing an empty string uses
-           a default configuration.
     \param scope_ini_file The full path of scope configuration file
     */
-    void run_scope(ScopeBase *const scope_base, std::string const& runtime_ini_file, std::string const& scope_ini_file);
-
-    /**
-    \brief Run a scope without going through the scope runner.
-
-    Equivalent to `run_scope(scope_base, "", scope_ini_file);`
-    */
     void run_scope(ScopeBase *const scope_base, std::string const& scope_ini_file);
 
     // TODO: Flesh out documentation for this, especially syntax.
@@ -175,5 +165,3 @@ private:
 } // namespace scopes
 
 } // namespace unity
-
-#endif
