@@ -16,8 +16,7 @@
  * Authored by: Michi Henning <michi.henning@canonical.com>
  */
 
-#ifndef UNITY_SCOPES_INTERNAL_MWQUERYPROXYFWD_H
-#define UNITY_SCOPES_INTERNAL_MWQUERYPROXYFWD_H
+#pragma once
 
 #include <memory>
 
@@ -38,5 +37,3 @@ typedef std::shared_ptr<MWQuery> MWQueryProxy;
 } // namespace scopes
 
 } // namespace unity
-
-#endif
