@@ -16,14 +16,14 @@
  * Authored by: Michi Henning <michi.henning@canonical.com>
  */
 
-#ifndef UNITY_SCOPES_INTERNAL_MWSCOPE_H
-#define UNITY_SCOPES_INTERNAL_MWSCOPE_H
+#pragma once
 
 #include <unity/scopes/internal/MWObjectProxy.h>
 #include <unity/scopes/internal/MWReplyProxyFwd.h>
 #include <unity/scopes/internal/MWScopeProxyFwd.h>
-#include <unity/scopes/QueryCtrlProxyFwd.h>
 #include <unity/scopes/CannedQuery.h>
+#include <unity/scopes/ChildScope.h>
+#include <unity/scopes/QueryCtrlProxyFwd.h>
 #include <unity/scopes/Variant.h>
 
 namespace unity
@@ -40,10 +40,28 @@ class MWScope : public virtual MWObjectProxy
 public:
     virtual ~MWScope();
 
-    virtual QueryCtrlProxy search(CannedQuery const& query, VariantMap const& hints, MWReplyProxy const& reply) = 0;
-    virtual QueryCtrlProxy activate(VariantMap const& result, VariantMap const& hints, MWReplyProxy const& reply) = 0;
-    virtual QueryCtrlProxy perform_action(VariantMap const& result, VariantMap const& hints, std::string const& widget_id, std::string const& action_id, MWReplyProxy const& reply) = 0;
-    virtual QueryCtrlProxy preview(VariantMap const& result, VariantMap const& hints, MWReplyProxy const& reply) = 0;
+    virtual QueryCtrlProxy search(CannedQuery const& query,
+                                  VariantMap const& hints,
+                                  VariantMap const& details,
+                                  MWReplyProxy const& reply) = 0;
+
+    virtual QueryCtrlProxy activate(VariantMap const& result,
+                                    VariantMap const& hints,
+                                    MWReplyProxy const& reply) = 0;
+
+    virtual QueryCtrlProxy perform_action(VariantMap const& result,
+                                          VariantMap const& hints,
+                                          std::string const& widget_id,
+                                          std::string const& action_id,
+                                          MWReplyProxy const& reply) = 0;
+
+    virtual QueryCtrlProxy preview(VariantMap const& result,
+                                   VariantMap const& hints,
+                                   MWReplyProxy const& reply) = 0;
+
+    virtual ChildScopeList child_scopes() = 0;
+    virtual bool set_child_scopes(ChildScopeList const& child_scopes) = 0;
+
     virtual bool debug_mode() = 0;
 
 protected:
@@ -55,5 +73,3 @@ protected:
 } // namespace scopes
 
 } // namespace unity
-
-#endif
