@@ -16,14 +16,14 @@
  * Authored by: Marcus Tomlinson <marcus.tomlinson@canonical.com>
  */
 
-#ifndef UNITY_SCOPES_INTERNAL_REGISTRYOBJECT_H
-#define UNITY_SCOPES_INTERNAL_REGISTRYOBJECT_H
+#pragma once
 
 #include <unity/scopes/internal/Executor.h>
 #include <unity/scopes/internal/MiddlewareBase.h>
 #include <unity/scopes/internal/MWPublisher.h>
 #include <unity/scopes/internal/MWRegistryProxyFwd.h>
 #include <unity/scopes/internal/RegistryObjectBase.h>
+#include <unity/scopes/internal/RuntimeImpl.h>
 #include <unity/scopes/internal/StateReceiverObject.h>
 
 #include <condition_variable>
@@ -58,6 +58,7 @@ public:
 
 public:
     UNITY_DEFINES_PTRS(RegistryObject);
+    NONCOPYABLE(RegistryObject);
 
     RegistryObject(core::posix::ChildProcess::DeathObserver& death_observer, Executor::SPtr const& executor,
                    MiddlewareBase::SPtr middleware, bool generate_desktop_files = false);
@@ -85,16 +86,21 @@ private:
     void create_desktop_file(ScopeMetadata const& metadata);
     void remove_desktop_file(std::string const& scope_id);
 
+    void ss_list_update();
+
     class ScopeProcess
     {
     public:
+        NONCOPYABLE(ScopeProcess);
+
         enum ProcessState
         {
             Stopped, Starting, Running, Stopping
         };
 
-        ScopeProcess(ScopeExecData exec_data, std::weak_ptr<MWPublisher> const& publisher);
-        ScopeProcess(ScopeProcess const& other);
+        ScopeProcess(ScopeExecData exec_data,
+                     std::weak_ptr<MWPublisher> const& publisher,
+                     boost::log::sources::severity_channel_logger_mt<>& logger);
         ~ScopeProcess();
 
         ProcessState state() const;
@@ -118,7 +124,6 @@ private:
         std::vector<std::string> expand_custom_exec();
         void publish_state_change(ProcessState scope_state);
 
-    private:
         const ScopeExecData exec_data_;
         ProcessState state_ = Stopped;
         mutable std::mutex process_mutex_;
@@ -126,9 +131,12 @@ private:
         core::posix::ChildProcess process_ = core::posix::ChildProcess::invalid();
         std::weak_ptr<MWPublisher> reg_publisher_; // weak_ptr, so processes don't hold publisher alive
         bool manually_started_;
+        boost::log::sources::severity_channel_logger_mt<>& logger_;
     };
 
 private:
+    boost::log::sources::severity_channel_logger_mt<>& logger_;
+
     core::posix::ChildProcess::DeathObserver& death_observer_;
     core::ScopedConnection death_observer_connection_;
 
@@ -138,12 +146,14 @@ private:
     Executor::SPtr executor_;
 
     MetadataMap scopes_;
-    typedef std::map<std::string, ScopeProcess> ProcessMap;
+    typedef std::map<std::string, std::shared_ptr<ScopeProcess>> ProcessMap;
     ProcessMap scope_processes_;
     MWRegistryProxy remote_registry_;
     mutable std::mutex mutex_;
 
     MWPublisher::SPtr publisher_;
+    MWSubscriber::SPtr ss_list_update_subscriber_;
+    std::shared_ptr<core::ScopedConnection> ss_list_update_connection_;
     bool generate_desktop_files_;
 };
 
@@ -152,5 +162,3 @@ private:
 } // namespace scopes
 
 } // namespace unity
-
-#endif
