@@ -16,8 +16,7 @@
  * Authored by: Michi Henning <michi.henning@canonical.com>
  */
 
-#ifndef UNITY_SCOPES_INTERNAL_REPLYOBJECT_H
-#define UNITY_SCOPES_INTERNAL_REPLYOBJECT_H
+#pragma once
 
 #include <unity/scopes/internal/ReplyObjectBase.h>
 #include <unity/scopes/internal/Reaper.h>
@@ -36,6 +35,7 @@ namespace scopes
 namespace internal
 {
 
+struct InvokeInfo;
 class RuntimeImpl;
 
 // A ReplyObject sits in between the incoming requests from the middleware layer and the
@@ -61,6 +61,7 @@ public:
     void info(OperationInfo const& op_info) noexcept override;
 
 private:
+    RuntimeImpl const* runtime_;
     ListenerBase::SPtr listener_base_;
     ReapItem::SPtr reap_item_;
     std::atomic_bool finished_;
@@ -68,7 +69,6 @@ private:
     std::condition_variable idle_;
     std::string origin_proxy_;
     int num_push_;
-    std::atomic_bool info_occurred_;
     std::vector<OperationInfo> info_list_;
 };
 
@@ -77,5 +77,3 @@ private:
 } // namespace scopes
 
 } // namespace unity
-
-#endif
