@@ -16,8 +16,7 @@
  * Authored by: Michi Henning <michi.henning@canonical.com>
  */
 
-#ifndef UNITY_SCOPES_INTERNAL_RESULTREPLYOBJECT_H
-#define UNITY_SCOPES_INTERNAL_RESULTREPLYOBJECT_H
+#pragma once
 
 #include <unity/scopes/internal/ReplyObject.h>
 #include <unity/scopes/internal/CategoryRegistry.h>
@@ -56,5 +55,3 @@ private:
 } // namespace scopes
 
 } // namespace unity
-
-#endif
