@@ -16,9 +16,9 @@
  * Authored by: Michi Henning <michi.henning@canonical.com>
  */
 
-#ifndef UNITY_SCOPES_INTERNAL_RUNTIMEIMPL_H
-#define UNITY_SCOPES_INTERNAL_RUNTIMEIMPL_H
+#pragma once
 
+#include <unity/scopes/internal/Logger.h>
 #include <unity/scopes/internal/MiddlewareBase.h>
 #include <unity/scopes/internal/MiddlewareFactory.h>
 #include <unity/scopes/internal/Reaper.h>
@@ -53,38 +53,49 @@ public:
     Reaper::SPtr reply_reaper() const;
     ThreadPool::SPtr async_pool() const;
     ThreadSafeQueue<std::future<void>>::SPtr future_queue() const;
+    boost::log::sources::severity_channel_logger_mt<>& logger() const;
+    boost::log::sources::severity_channel_logger_mt<>& logger(Logger::Channel channel) const;
     void run_scope(ScopeBase* scope_base,
                    std::string const& scope_ini_file,
                    std::promise<void> ready_promise = std::promise<void>());
-    void run_scope(ScopeBase* scope_base,
-                   std::string const& runtime_ini_file,
-                   std::string const& scope_ini_file,
-                   std::promise<void> = std::promise<void>());
 
     ObjectProxy string_to_proxy(std::string const& s) const;
     std::string proxy_to_string(ObjectProxy const& proxy) const;
 
+    std::string cache_directory() const;
+    std::string tmp_directory() const;
+    std::string config_directory() const;
+
     ~RuntimeImpl();
 
 private:
     RuntimeImpl(std::string const& scope_id, std::string const& configfile);
     void waiter_thread(ThreadSafeQueue<std::future<void>>::SPtr const& queue) const noexcept;
-    std::string find_cache_dir(std::string& confinement_type) const;
+    std::string demangled_id(std::string const& scope_id) const;
+    bool confined() const;
+    std::string confinement_type() const;
+    std::string find_cache_dir() const;
+    std::string find_app_dir() const;
+    std::string find_log_dir(std::string const& id) const;
+    std::string find_tmp_dir() const;
 
     bool destroyed_;
     std::string scope_id_;
     MiddlewareFactory::UPtr middleware_factory_;
     MiddlewareBase::SPtr middleware_;
-    mutable RegistryProxy registry_;
-    mutable std::string registry_configfile_;
-    mutable std::string registry_identity_;
-    mutable std::string ss_configfile_;
-    mutable std::string ss_registry_identity_;
+    RegistryProxy registry_;
+    std::string runtime_configfile_;
+    std::string registry_configfile_;
+    std::string registry_identity_;
+    std::string ss_configfile_;
+    std::string ss_registry_identity_;
     int reap_expiry_;
     int reap_interval_;
-    std::string data_dir_;
+    std::string cache_dir_;
+    std::string app_dir_;
+    std::string log_dir_;
     std::string config_dir_;
-    std::string tmp_dir_;
+    Logger::UPtr logger_;
     mutable Reaper::SPtr reply_reaper_;
     mutable ThreadPool::SPtr async_pool_;  // Pool of invocation threads for async query creation
     mutable ThreadSafeQueue<std::future<void>>::SPtr future_queue_;
@@ -97,5 +108,3 @@ private:
 } // namespace scopes
 
 } // namespace unity
-
-#endif
