@@ -16,8 +16,7 @@
  * Authored by: Michi Henning <michi.henning@canonical.com>
  */
 
-#ifndef UNITY_SCOPES_INTERNAL_SCOPEOBJECT_H
-#define UNITY_SCOPES_INTERNAL_SCOPEOBJECT_H
+#pragma once
 
 #include <unity/scopes/internal/ScopeObjectBase.h>
 #include <unity/scopes/internal/QueryObjectBase.h>
@@ -41,7 +40,6 @@ namespace internal
 {
 
 class MiddlewareBase;
-class RuntimeImpl;
 
 // A ScopeObject sits in between the incoming requests from the middleware layer and the
 // ScopeBase-derived implementation provided by the scope. It forwards incoming
@@ -52,19 +50,20 @@ class ScopeObject final : public ScopeObjectBase
 public:
     UNITY_DEFINES_PTRS(ScopeObject);
 
-    ScopeObject(RuntimeImpl* runtime, ScopeBase* scope_base, bool debug_mode = false);
+    ScopeObject(ScopeBase* scope_base, bool debug_mode = false);
     virtual ~ScopeObject();
 
     // Remote operation implementations
     virtual MWQueryCtrlProxy search(CannedQuery const& q,
-                                          SearchMetadata const& hints,
-                                          MWReplyProxy const& reply,
-                                          InvokeInfo const& info) override;
+                                    SearchMetadata const& hints,
+                                    VariantMap const& context,
+                                    MWReplyProxy const& reply,
+                                    InvokeInfo const& info) override;
 
     virtual MWQueryCtrlProxy activate(Result const& result,
-                              ActionMetadata const& hints,
-                              MWReplyProxy const &reply,
-                              InvokeInfo const& info) override;
+                                      ActionMetadata const& hints,
+                                      MWReplyProxy const &reply,
+                                      InvokeInfo const& info) override;
 
     virtual MWQueryCtrlProxy perform_action(Result const& result,
                                             ActionMetadata const& hints,
@@ -78,13 +77,16 @@ public:
                                      MWReplyProxy const& reply,
                                      InvokeInfo const& info) override;
 
+    virtual ChildScopeList child_scopes() const override;
+    virtual bool set_child_scopes(ChildScopeList const& child_scopes) override;
+
     virtual bool debug_mode() const override;
 
 private:
     MWQueryCtrlProxy query(MWReplyProxy const& reply, MiddlewareBase* mw_base,
+        std::string const& method,
         std::function<QueryBase::SPtr(void)> const& query_factory_fun,
         std::function<QueryObjectBase::SPtr(QueryBase::SPtr, MWQueryCtrlProxy)> const& query_object_factory_fun);
-    RuntimeImpl* const runtime_;
     ScopeBase* const scope_base_;
     bool const debug_mode_;
 };
@@ -94,5 +96,3 @@ private:
 } // namespace scopes
 
 } // namespace unity
-
-#endif
