@@ -16,13 +16,12 @@
  * Authored by: Michi Henning <michi.henning@canonical.com>
  */
 
-#ifndef UNITY_SCOPES_INTERNAL_SCOPEOBJECTBASE_H
-#define UNITY_SCOPES_INTERNAL_SCOPEOBJECTBASE_H
+#pragma once
 
 #include <unity/scopes/internal/AbstractObject.h>
-#include <unity/scopes/internal/InvokeInfo.h>
 #include <unity/scopes/internal/MWQueryCtrlProxyFwd.h>
 #include <unity/scopes/internal/MWReplyProxyFwd.h>
+#include <unity/scopes/ChildScope.h>
 #include <unity/scopes/Result.h>
 #include <unity/scopes/Variant.h>
 
@@ -40,8 +39,7 @@ class CannedQuery;
 namespace internal
 {
 
-class MiddlewareBase;
-class RuntimeImpl;
+struct InvokeInfo;
 
 class ScopeObjectBase : public AbstractObject
 {
@@ -50,6 +48,7 @@ public:
 
     virtual MWQueryCtrlProxy search(CannedQuery const& query,
                                           SearchMetadata const& hints,
+                                          VariantMap const& context,
                                           MWReplyProxy const& reply,
                                           InvokeInfo const& info) = 0;
 
@@ -70,6 +69,9 @@ public:
                                      MWReplyProxy const& reply,
                                      InvokeInfo const& info) = 0;
 
+    virtual ChildScopeList child_scopes() const = 0;
+    virtual bool set_child_scopes(ChildScopeList const& child_scopes) = 0;
+
     virtual bool debug_mode() const = 0;
 };
 
@@ -78,5 +80,3 @@ public:
 } // namespace scopes
 
 } // namespace unity
-
-#endif
