@@ -16,8 +16,7 @@
  * Authored by: Marcus Tomlinson <marcus.tomlinson@canonical.com>
  */
 
-#ifndef UNITY_SCOPES_INTERNAL_SMARTSCOPES_SSSCOPEOBJECT_H
-#define UNITY_SCOPES_INTERNAL_SMARTSCOPES_SSSCOPEOBJECT_H
+#pragma once
 
 #include <unity/scopes/internal/ScopeObjectBase.h>
 #include <unity/scopes/internal/smartscopes/SSQueryCtrlObject.h>
@@ -53,9 +52,10 @@ public:
 
     // Remote operation implementations
     MWQueryCtrlProxy search(CannedQuery const& q,
-                                  SearchMetadata const& hints,
-                                  MWReplyProxy const& reply,
-                                  InvokeInfo const& info) override;
+                            SearchMetadata const& hints,
+                            VariantMap const& context,
+                            MWReplyProxy const& reply,
+                            InvokeInfo const& info) override;
 
     MWQueryCtrlProxy activate(Result const& result,
                               ActionMetadata const& hints,
@@ -74,6 +74,9 @@ public:
                              MWReplyProxy const& reply,
                              InvokeInfo const& info) override;
 
+    ChildScopeList child_scopes() const override;
+    bool set_child_scopes(ChildScopeList const& child_scopes) override;
+
     bool debug_mode() const override;
 
 private:
@@ -82,7 +85,6 @@ private:
                            std::function<QueryBase::SPtr(void)> const& query_factory_fun,
                            std::function<void(QueryBase::SPtr)> const& query_object_fun);
 
-private:
     std::string ss_scope_id_;
 
     SSQueryCtrlObject::SPtr co_;
@@ -99,5 +101,3 @@ private:
 }  // namespace scopes
 
 }  // namespace unity
-
-#endif  // UNITY_SCOPES_INTERNAL_SMARTSCOPES_SSSCOPEOBJECT_H
