@@ -16,13 +16,13 @@
  * Authored by: Marcus Tomlinson <marcus.tomlinson@canonical.com>
  */
 
-#ifndef UNITY_SCOPES_INTERNAL_SMARTSCOPES_SMARTSCOPESCLIENT_H
-#define UNITY_SCOPES_INTERNAL_SMARTSCOPES_SMARTSCOPESCLIENT_H
+#pragma once
 
 #include <unity/scopes/FilterBase.h>
 #include <unity/scopes/FilterState.h>
-#include <unity/scopes/internal/smartscopes/HttpClientInterface.h>
 #include <unity/scopes/internal/JsonNodeInterface.h>
+#include <unity/scopes/internal/Logger.h>
+#include <unity/scopes/internal/smartscopes/HttpClientInterface.h>
 #include <unity/scopes/internal/UniqueID.h>
 
 #include <unity/util/NonCopyable.h>
@@ -43,6 +43,8 @@ namespace scopes
 namespace internal
 {
 
+class RuntimeImpl;
+
 namespace smartscopes
 {
 
@@ -61,6 +63,8 @@ struct RemoteScope
     std::shared_ptr<std::string> settings;      // optional
     std::shared_ptr<bool> needs_location_data;  // optional
     bool invisible = false;
+    int version;
+    std::set<std::string> keywords;             // optional
 };
 
 struct SearchCategory
@@ -88,6 +92,16 @@ struct DepartmentInfo
     std::vector<std::shared_ptr<DepartmentInfo>> subdepartments;
 };
 
+struct LocationInfo
+{
+    bool has_location;
+    std::string country_code;
+    double latitude;
+    double longitude;
+
+    LocationInfo(): has_location(false) {}
+};
+
 class SearchHandle
 {
 public:
@@ -154,7 +168,9 @@ public:
 
     SmartScopesClient(HttpClientInterface::SPtr http_client,
                       JsonNodeInterface::SPtr json_node,
-                      std::string const& url = ""); // detect url
+                      RuntimeImpl* runtime,
+                      std::string const& url = "", // detect url
+                      std::string const& partner_id_path = "");
 
     virtual ~SmartScopesClient();
 
@@ -173,7 +189,7 @@ public:
                               VariantMap const& settings = VariantMap(),
                               VariantMap const& filter_state = VariantMap(),
                               std::string const& locale = "",
-                              std::string const& country = "",
+                              LocationInfo const& location = LocationInfo(),
                               std::string const& user_agent_hdr = "",
                               const unsigned int limit = 0);
 
@@ -188,6 +204,8 @@ public:
                                 std::string const& country = "",
                                 std::string const& user_agent_hdr = "");
 
+    boost::log::sources::severity_channel_logger_mt<>& logger() const;
+
 private:
     friend class SearchHandle;
     friend class PreviewHandle;
@@ -209,10 +227,9 @@ private:
 
     std::string stringify_settings(VariantMap const& settings);
 
-private:
     HttpClientInterface::SPtr http_client_;
     JsonNodeInterface::SPtr json_node_;
-
+    boost::log::sources::severity_channel_logger_mt<>& logger_;
     std::string url_;
 
     std::map<unsigned int, HttpResponseHandle::SPtr> query_results_;
@@ -224,6 +241,7 @@ private:
     bool have_latest_cache_;
 
     unsigned int query_counter_;
+    std::string partner_file_;
 };
 
 }  // namespace smartscopes
@@ -233,5 +251,3 @@ private:
 }  // namespace scopes
 
 }  // namespace unity
-
-#endif  // UNITY_SCOPES_INTERNAL_SMARTSCOPES_SMARTSCOPESCLIENT_H
