@@ -16,8 +16,7 @@
  * Authored by: Michi Henning <michi.henning@canonical.com>
  */
 
-#ifndef UNITY_SCOPES_INTERNAL_ZMQMIDDLEWARE_SCOPEI_H
-#define UNITY_SCOPES_INTERNAL_ZMQMIDDLEWARE_SCOPEI_H
+#pragma once
 
 #include <unity/scopes/internal/MWScope.h>
 #include <unity/scopes/internal/ScopeObjectBase.h>
@@ -54,6 +53,12 @@ private:
     virtual void preview_(Current const& current,
                           capnp::AnyPointer::Reader& in_params,
                           capnproto::Response::Builder& r);
+    virtual void child_scopes_(Current const& current,
+                          capnp::AnyPointer::Reader& in_params,
+                          capnproto::Response::Builder& r);
+    virtual void set_child_scopes_(Current const& current,
+                          capnp::AnyPointer::Reader& in_params,
+                          capnproto::Response::Builder& r);
     virtual void debug_mode_(Current const& current,
                              capnp::AnyPointer::Reader& in_params,
                              capnproto::Response::Builder& r);
@@ -66,5 +71,3 @@ private:
 } // namespace scopes
 
 } // namespace unity
-
-#endif
