@@ -16,8 +16,7 @@
  * Authored by: Michi Henning <michi.henning@canonical.com>
  */
 
-#ifndef UNITY_SCOPES_INTERNAL_ZMQMIDDLEWARE_ZMQSCOPE_H
-#define UNITY_SCOPES_INTERNAL_ZMQMIDDLEWARE_ZMQSCOPE_H
+#pragma once
 
 #include <unity/scopes/internal/zmq_middleware/ZmqObjectProxy.h>
 #include <unity/scopes/internal/zmq_middleware/ZmqScopeProxyFwd.h>
@@ -46,8 +45,9 @@ public:
     virtual ~ZmqScope();
 
     virtual QueryCtrlProxy search(CannedQuery const& query,
-                                        VariantMap const& hints,
-                                        MWReplyProxy const& reply) override;
+                                  VariantMap const& hints,
+                                  VariantMap const& context,
+                                  MWReplyProxy const& reply) override;
 
     virtual QueryCtrlProxy activate(VariantMap const& result,
                                     VariantMap const& hints,
@@ -63,6 +63,9 @@ public:
                                    VariantMap const& hints,
                                    MWReplyProxy const& reply) override;
 
+    virtual ChildScopeList child_scopes() override;
+    virtual bool set_child_scopes(ChildScopeList const& child_scopes) override;
+
     virtual bool debug_mode() override;
 
 private:
@@ -78,5 +81,3 @@ private:
 } // namespace scopes
 
 } // namespace unity
-
-#endif
