@@ -20,10 +20,7 @@
 #include "ScopesWatcher.h"
 
 #include <unity/scopes/internal/IniSettingsSchema.h>
-#include <unity/scopes/internal/MiddlewareFactory.h>
-#include <unity/scopes/internal/MWRegistry.h>
 #include <unity/scopes/internal/RegistryConfig.h>
-#include <unity/scopes/internal/RegistryObject.h>
 #include <unity/scopes/internal/RuntimeConfig.h>
 #include <unity/scopes/internal/RuntimeImpl.h>
 #include <unity/scopes/internal/ScopeConfig.h>
@@ -31,19 +28,12 @@
 #include <unity/scopes/internal/ScopeMetadataImpl.h>
 #include <unity/scopes/ScopeExceptions.h>
 #include <unity/UnityExceptions.h>
-#include <unity/util/ResourcePtr.h>
 
 #include <boost/algorithm/string.hpp>
 #include <boost/filesystem/operations.hpp>
 
-#include <algorithm>
-#include <cassert>
-#include <fstream>
-#include <iostream>
-#include <map>
-#include <set>
-
 #include <sys/stat.h>  // TODO: remove this once hack for creating data root dir is removed
+#include <wordexp.h>
 
 using namespace scoperegistry;
 using namespace std;
@@ -77,35 +67,35 @@ filesystem::path relative_scope_path_to_abs_path(filesystem::path const& path, f
 // throwing an exception without joining with a thread is a bad, bad thing to do, so let's RAII to avoid doing it
 struct SignalThreadWrapper
 {
-    std::shared_ptr<core::posix::SignalTrap> termination_trap;
-    std::shared_ptr<core::posix::SignalTrap> child_trap;
-    std::unique_ptr<core::posix::ChildProcess::DeathObserver> death_observer;
+    std::shared_ptr<::core::posix::SignalTrap> termination_trap;
+    std::shared_ptr<::core::posix::SignalTrap> child_trap;
+    std::unique_ptr<::core::posix::ChildProcess::DeathObserver> death_observer;
     std::thread termination_trap_worker;
     std::thread child_trap_worker;
 
     SignalThreadWrapper() :
         // We shutdown the runtime whenever these signals happen.
-        termination_trap(core::posix::trap_signals_for_all_subsequent_threads(
+        termination_trap(::core::posix::trap_signals_for_all_subsequent_threads(
             {
-                core::posix::Signal::sig_int,
-                core::posix::Signal::sig_hup,
-                core::posix::Signal::sig_term
+                ::core::posix::Signal::sig_int,
+                ::core::posix::Signal::sig_hup,
+                ::core::posix::Signal::sig_term
             })),
         // And we maintain our list of processes with the help of sig_chld.
-        child_trap(core::posix::trap_signals_for_all_subsequent_threads(
+        child_trap(::core::posix::trap_signals_for_all_subsequent_threads(
             {
-                core::posix::Signal::sig_chld
+                ::core::posix::Signal::sig_chld
             })),
         // The death observer is required to make sure that we reap all child processes
         // whenever multiple sigchld's are compressed together.
-        death_observer(core::posix::ChildProcess::DeathObserver::create_once_with_signal_trap(child_trap)),
+        death_observer(::core::posix::ChildProcess::DeathObserver::create_once_with_signal_trap(child_trap)),
         // Starting up both traps.
         termination_trap_worker([&]() { termination_trap->run(); }),
         child_trap_worker([&]() { child_trap->run(); })
     {
     }
 
-    core::Signal<core::posix::Signal>& signal_raised()
+    ::core::Signal<::core::posix::Signal>& signal_raised()
     {
         return termination_trap->signal_raised();
     }
@@ -183,7 +173,7 @@ map<string, string> find_local_scopes(string const& scope_installdir, string con
                 }
             }
         }
-        catch (ResourceException const& e)
+        catch (FileException const& e)
         {
             error(e.what());
             error("could not open OEM installation directory, ignoring OEM scopes");
@@ -221,7 +211,7 @@ map<string, string> find_click_scopes(map<string, string> const& local_scopes, s
                 }
             }
         }
-        catch (ResourceException const& e)
+        catch (FileException const& e)
         {
             error(e.what());
             error("could not open Click installation directory, ignoring Click scopes");
@@ -269,6 +259,30 @@ void convert_relative_attribute(VariantMap& inner_map, const string& id, const f
     }
 }
 
+string get_first_component(string const& id, string const& custom_exec)
+{
+    if (custom_exec.empty())
+    {
+        throw unity::InvalidArgumentException("Invalid scope runner executable for scope: " + id);
+    }
+
+    wordexp_t exp;
+    string result;
+
+    // Split command into program and args
+    if (wordexp(custom_exec.c_str(), &exp, WRDE_NOCMD) == 0)
+    {
+        util::ResourcePtr<wordexp_t*, decltype(&wordfree)> free_guard(&exp, wordfree);
+        result = exp.we_wordv[0];
+    }
+    else
+    {
+        throw unity::InvalidArgumentException("Invalid scope runner executable for scope: " + id);
+    }
+
+    return result;
+}
+
 // For each scope, open the config file for the scope, create the metadata info from the config,
 // and add an entry to the RegistryObject.
 // If the scope uses settings, also parse the settings file and add the settings to the metadata.
@@ -299,9 +313,11 @@ void add_local_scope(RegistryObject::SPtr const& registry,
         {
             schema = IniSettingsSchema::create(settings_schema_path.native());
         }
-        // We always need to create a settings schema if the scope wants location data
         else if (sc.location_data_needed())
         {
+            // TODO: HACK: See bug #1393438 and the comments in IniSettingsSchema.cpp and
+            //             JsonSettingsSchema.cpp.
+            // We always need to create a settings schema if the scope wants location data
             schema = IniSettingsSchema::create_empty();
         }
 
@@ -326,6 +342,10 @@ void add_local_scope(RegistryObject::SPtr const& registry,
     mi->set_author(sc.author());
     mi->set_invisible(sc.invisible());
     mi->set_appearance_attributes(sc.appearance_attributes());
+    mi->set_child_scope_ids(sc.child_scope_ids());
+    mi->set_version(sc.version());
+    mi->set_keywords(sc.keywords());
+    mi->set_is_aggregator(sc.is_aggregator());
 
     // Prepend scope_dir to pageheader logo path if logo path is relative.
     // TODO: Once we have type-safe parsing in the config files, remove
@@ -388,9 +408,9 @@ void add_local_scope(RegistryObject::SPtr const& registry,
     {
     }
 
-    ScopeProxy proxy = ScopeImpl::create(mw->create_scope_proxy(scope.first), mw->runtime(), scope.first);
+    ScopeProxy proxy = ScopeImpl::create(mw->create_scope_proxy(scope.first), scope.first);
     mi->set_proxy(proxy);
-    auto meta = ScopeMetadataImpl::create(move(mi));
+    auto meta = ScopeMetadataImpl::create(std::move(mi));
 
     RegistryObject::ScopeExecData exec_data;
     exec_data.scope_id = scope.first;
@@ -416,11 +436,32 @@ void add_local_scope(RegistryObject::SPtr const& registry,
     try
     {
         auto custom_exec = sc.scope_runner();
-        if (custom_exec.empty())
+        filesystem::path program(get_first_component( scope.first, custom_exec));
+        if (program.is_relative())
         {
-            throw unity::InvalidArgumentException("Invalid scope runner executable for scope: " + scope.first);
+            // First look inside the arch-specific directory
+            if (filesystem::exists(scope_dir / DEB_HOST_MULTIARCH / program))
+            {
+                // Join the full command, not just the program path
+                exec_data.custom_exec = (scope_dir / DEB_HOST_MULTIARCH / custom_exec).native();
+            }
+            // Next try in the non arch-aware directory
+            else if (filesystem::exists(scope_dir / program))
+            {
+                // Join the full command, not just the program path
+                exec_data.custom_exec = (scope_dir / custom_exec).native();
+            }
+            else
+            {
+                throw unity::InvalidArgumentException(
+                        "Nonexistent scope runner '" + custom_exec
+                                + "' executable for scope: " + scope.first);
+            }
+        }
+        else
+        {
+            exec_data.custom_exec = custom_exec;
         }
-        exec_data.custom_exec = relative_scope_path_to_abs_path(custom_exec, scope_dir).native();
     }
     catch (NotFoundException const&)
     {
@@ -429,7 +470,7 @@ void add_local_scope(RegistryObject::SPtr const& registry,
     exec_data.scope_config = scope.second;
     exec_data.debug_mode = sc.debug_mode();
 
-    registry->add_local_scope(scope.first, move(meta), exec_data);
+    registry->add_local_scope(scope.first, std::move(meta), exec_data);
 }
 
 void add_local_scopes(RegistryObject::SPtr const& registry,
@@ -486,7 +527,7 @@ int main(int argc, char* argv[])
         // And finally creating our runtime.
         string identity;
         string ss_reg_id;
-        RuntimeImpl::UPtr runtime;
+        RuntimeImpl::SPtr runtime;
         {
             RuntimeConfig rt_config(config_file);
             runtime = RuntimeImpl::create(rt_config.registry_identity(), config_file);
@@ -494,16 +535,29 @@ int main(int argc, char* argv[])
             identity = runtime->registry_identity();
             ss_reg_id = runtime->ss_registry_identity();
 
-            // TODO: HACK: We create the root of the data directory for confined scopes,
-            //       in case the scope is confined and the dir doesn't exist
-            //       yet. This really should be done by the click-installation but,
-            //       prior to RTM, we don't rely on that. We check whether the
-            //       directory exists first to avoid getting noise in the Apparmor log.
-            string data_root = rt_config.data_directory() + "/leaf-net";
+            // TODO: HACK: We create the root of the cache and app directories for
+            //       confined scopes, in case the scope is confined and the dir doesn't
+            //       exist yet. This really should be done by the click-installation but,
+            //       prior to RTM, we don't rely on that.
             boost::system::error_code ec;
-            !boost::filesystem::exists(data_root, ec) && ::mkdir(data_root.c_str(), 0700);
+
+            string cache_root = rt_config.cache_directory() + "/leaf-net";
+            !boost::filesystem::exists(cache_root, ec) && ::mkdir(cache_root.c_str(), 0700);
+
+            string app_root = rt_config.app_directory();
+            !boost::filesystem::exists(app_root, ec) && ::mkdir(app_root.c_str(), 0700);
         } // Release memory for config parser
 
+        // Make sure that the parent directories for confined scope tmp directory exist.
+        {
+            string dir = string("/run/user/") + std::to_string(geteuid());
+            dir += "/scopes";
+            boost::system::error_code ec;
+            !boost::filesystem::exists(dir, ec) && ::mkdir(dir.c_str(), 0700 | S_ISVTX);
+            dir += "/leaf-net";
+            !boost::filesystem::exists(dir, ec) && ::mkdir(dir.c_str(), 0700 | S_ISVTX);
+        }
+
         // Collect the registry config data.
 
         string mw_kind;
@@ -522,18 +576,16 @@ int main(int argc, char* argv[])
             process_timeout = c.process_timeout();
         } // Release memory for config parser
 
-        MiddlewareBase::SPtr middleware = runtime->factory()->find(identity, mw_kind);
-
-        // Inform the signal thread that it should shutdown the middleware
+        // Inform the signal thread that it should shutdown the runtime
         // if we get a termination signal.
-        signal_handler_wrapper.signal_raised().connect([middleware](core::posix::Signal signal)
+        signal_handler_wrapper.signal_raised().connect([runtime](::core::posix::Signal signal)
         {
             switch(signal)
             {
-            case core::posix::Signal::sig_int:
-            case core::posix::Signal::sig_hup:
-            case core::posix::Signal::sig_term:
-                middleware->stop();
+            case ::core::posix::Signal::sig_int:
+            case ::core::posix::Signal::sig_hup:
+            case ::core::posix::Signal::sig_term:
+                runtime->destroy();
                 break;
             default:
                 break;
@@ -541,7 +593,8 @@ int main(int argc, char* argv[])
         });
 
         // The registry object stores the local and remote scopes
-        Executor::SPtr executor = make_shared<Executor>();
+        MiddlewareBase::SPtr middleware = runtime->factory()->find(identity, mw_kind);
+        Executor::SPtr executor = std::make_shared<Executor>();
         RegistryObject::SPtr registry(new RegistryObject(*signal_handler_wrapper.death_observer, executor, middleware, true));
 
         // Add the metadata for each scope to the lookup table.
@@ -585,7 +638,7 @@ int main(int argc, char* argv[])
                 error("ignoring installed scope \"" + scope.first + "\": cannot create metadata: " + e.what());
             }
         };
-        ScopesWatcher local_scopes_watcher(registry, local_watch_lambda);
+        ScopesWatcher local_scopes_watcher(registry, local_watch_lambda, runtime->logger());
         local_scopes_watcher.add_install_dir(scope_installdir);
         local_scopes_watcher.add_install_dir(oem_installdir);
 
@@ -601,7 +654,7 @@ int main(int argc, char* argv[])
                 error("ignoring installed scope \"" + scope.first + "\": cannot create metadata: " + e.what());
             }
         };
-        ScopesWatcher click_scopes_watcher(registry, click_watch_lambda);
+        ScopesWatcher click_scopes_watcher(registry, click_watch_lambda, runtime->logger());
         click_scopes_watcher.add_install_dir(click_installdir);
 
         // Let's add the registry's state receiver to the middleware so that scopes can inform
