@@ -82,6 +82,11 @@ std::string ScopeBase::cache_directory() const
     return p->cache_directory();
 }
 
+std::string ScopeBase::app_directory() const
+{
+    return p->app_directory();
+}
+
 std::string ScopeBase::tmp_directory() const
 {
     return p->tmp_directory();
@@ -97,6 +102,16 @@ unity::scopes::VariantMap ScopeBase::settings() const
     return p->settings();
 }
 
+ChildScopeList ScopeBase::find_child_scopes() const
+{
+    return p->find_child_scopes();
+}
+
+ChildScopeList ScopeBase::child_scopes() const
+{
+    return p->child_scopes(find_child_scopes());
+}
+
 } // namespace scopes
 
 } // namespace unity
