@@ -26,6 +26,7 @@
 #include <unity/scopes/internal/RatingFilterImpl.h>
 #include <unity/scopes/internal/SwitchFilterImpl.h>
 #include <unity/scopes/internal/ValueSliderFilterImpl.h>
+#include <unity/scopes/internal/DateTimePickerFilterImpl.h>
 #include <unity/UnityExceptions.h>
 #include <sstream>
 
@@ -150,6 +151,10 @@ FilterBase::SCPtr FilterBaseImpl::deserialize(VariantMap const& var)
         {
             return ValueSliderFilterImpl::create(var);
         }
+        if (ftype == "datetime_picker")
+        {
+            return DateTimePickerFilterImpl::create(var);
+        }
         throw unity::LogicException("Unknown filter type: " + ftype);
     }
     throw unity::LogicException("FilterBase: Missing 'filter_type'");
