@@ -22,6 +22,9 @@
 
 #include <iostream>
 
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wold-style-cast"  // Lots of warnings from libg otherwise.
+
 namespace unity
 {
 
@@ -47,13 +50,15 @@ static void free_variant(GVariant* v)
     }
 }
 
-static OnlineAccountClient::ServiceStatus info_to_details(AccountInfo const* info, std::string const& error = "")
+static OnlineAccountClient::ServiceStatus info_to_details(AccountInfo* info, std::string const& error = "")
 {
     char* client_id = nullptr;
     char* client_secret = nullptr;
     char* access_token = nullptr;
     char* token_secret = nullptr;
 
+    std::lock_guard<std::mutex> info_lock(info->mutex);
+
     if (info->auth_params)
     {
         // Look up OAuth 2 parameters
@@ -115,16 +120,21 @@ static void service_login_cb(GObject* source, GAsyncResult* result, void* user_d
     SignonAuthSession* session = reinterpret_cast<SignonAuthSession*>(source);
     AccountInfo* info = reinterpret_cast<AccountInfo*>(user_data);
 
+    std::unique_lock<std::mutex> info_lock(info->mutex);
+
     // Get session data then send a notification with the login result
     GError* error = nullptr;
     info->session_data.reset(signon_auth_session_process_finish(session, result, &error), free_variant);
     std::shared_ptr<GError> error_cleanup(error, free_error);
 
+    info_lock.unlock();
     info->account_client->callback(info, error ? error->message : "");
 }
 
 static void service_update_cb(AgAccountService* account_service, gboolean enabled, AccountInfo* info)
 {
+    std::unique_lock<std::mutex> info_lock(info->mutex);
+
     // If another session is currently busy, cancel it
     if (info->session)
     {
@@ -147,13 +157,30 @@ static void service_update_cb(AgAccountService* account_service, gboolean enable
         if (error)
         {
             // Send notification that the authorization session failed
+            info_lock.unlock();                                    // LCOV_EXCL_LINE
             info->account_client->callback(info, error->message);  // LCOV_EXCL_LINE
             return;
         }
 
         // Get authorization parameters then attempt to signon
+        GVariantBuilder builder;
+        g_variant_builder_init(&builder, G_VARIANT_TYPE_VARDICT);
+
+        if (getenv("UNITY_SCOPES_OA_UI_POLICY"))
+        {
+            g_variant_builder_add(&builder, "{sv}",
+                                  SIGNON_SESSION_DATA_UI_POLICY,
+                                  g_variant_new_int32(SIGNON_POLICY_DEFAULT));  // LCOV_EXCL_LINE
+        }
+        else
+        {
+            g_variant_builder_add(&builder, "{sv}",
+                                  SIGNON_SESSION_DATA_UI_POLICY,
+                                  g_variant_new_int32(SIGNON_POLICY_NO_USER_INTERACTION));
+        }
+
         info->auth_params.reset(
-            g_variant_ref_sink(ag_auth_data_get_login_parameters(auth_data.get(), nullptr)), free_variant);
+            g_variant_ref_sink(ag_auth_data_get_login_parameters(auth_data.get(), g_variant_builder_end(&builder))), free_variant);
 
         // Start signon process
         signon_auth_session_process_async(info->session.get(),
@@ -166,6 +193,7 @@ static void service_update_cb(AgAccountService* account_service, gboolean enable
     else
     {
         // Send notification that account has been disabled
+        info_lock.unlock();
         info->account_client->callback(info);
     }
 }
@@ -232,11 +260,10 @@ OnlineAccountClientImpl::OnlineAccountClientImpl(std::string const& service_name
     if (main_loop_select_ == OnlineAccountClient::CreateInternalMainLoop)
     {
         // Wait here until either the main loop begins running, or the thread exits
-        {
-            std::unique_lock<std::mutex> lock(mutex_);
-            main_loop_thread_ = std::thread(&OnlineAccountClientImpl::main_loop_thread, this);
-            cond_.wait_for(lock, std::chrono::seconds(5), [this] { return main_loop_is_running_; });
-        }
+        std::unique_lock<std::mutex> lock(mutex_);
+        main_loop_thread_ = std::thread(&OnlineAccountClientImpl::main_loop_thread, this);
+        cond_.wait_for(lock, std::chrono::seconds(5), [this] { return main_loop_is_running_; });
+
         if (!main_loop_is_running_)
         {
             // LCOV_EXCL_START
@@ -247,7 +274,9 @@ OnlineAccountClientImpl::OnlineAccountClientImpl(std::string const& service_name
             }
             if (main_loop_thread_.joinable())
             {
+                lock.unlock();
                 main_loop_thread_.join();
+                lock.lock();
             }
             if (thread_exception_)
             {
@@ -272,6 +301,10 @@ OnlineAccountClientImpl::~OnlineAccountClientImpl()
         std::lock_guard<std::mutex> lock(mutex_);
         if (thread_exception_)
         {
+            // TODO: We don't have access to the run time here, so we can't write to the log.
+            //       It would be better to have a factory method on Runtime that instantiates
+            //       an OnlineAccoutClient, in which case we could access the logger here.
+            //       But that's an ABI-breaking change...
             // LCOV_EXCL_START
             try
             {
@@ -293,17 +326,20 @@ OnlineAccountClientImpl::~OnlineAccountClientImpl()
     if (main_loop_select_ == OnlineAccountClient::CreateInternalMainLoop)
     {
         // Invoke tear_down() from within the main loop
-        {
-            std::unique_lock<std::mutex> lock(mutex_);
-            g_main_context_invoke(main_loop_context_.get(), main_loop_is_stopping_cb, this);
-            cond_.wait(lock, [this] { return !main_loop_is_running_; });
-        }
+        std::unique_lock<std::mutex> lock(mutex_);
+        g_main_context_invoke(main_loop_context_.get(), main_loop_is_stopping_cb, this);
+        cond_.wait(lock, [this] { return !main_loop_is_running_; });
 
-        // Quit the main loop, causing the thread to exit
-        g_main_loop_quit(main_loop_.get());
+        if (main_loop_)
+        {
+            // Quit the main loop, causing the thread to exit
+            g_main_loop_quit(main_loop_.get());
+        }
         if (main_loop_thread_.joinable())
         {
+            lock.unlock();
             main_loop_thread_.join();
+            lock.lock();
         }
     }
     else
@@ -322,7 +358,7 @@ OnlineAccountClientImpl::~OnlineAccountClientImpl()
 
 void OnlineAccountClientImpl::set_service_update_callback(OnlineAccountClient::ServiceUpdateCallback callback)
 {
-    std::lock_guard<std::mutex> lock(mutex_);
+    std::unique_lock<std::mutex> lock(mutex_);
     if (thread_exception_)
     {
         std::rethrow_exception(thread_exception_);  // LCOV_EXCL_LINE
@@ -330,6 +366,30 @@ void OnlineAccountClientImpl::set_service_update_callback(OnlineAccountClient::S
 
     std::lock_guard<std::mutex> callback_lock(callback_mutex_);
     callback_ = callback;
+
+    if (callback_ != nullptr)
+    {
+        std::vector<OnlineAccountClient::ServiceStatus> known_statuses;
+        for (auto const& info : accounts_)
+        {
+            // We only want to invoke the callback for non-busy sessions, as signon sessions that are currently
+            // busy will end with a callback anyway. info->session is cleared once a signon process has fully
+            // completed, therefore, if the session is null, we know that it is not currently busy.
+            std::unique_lock<std::mutex> info_lock(info.second->mutex);
+            if (info.second->session == nullptr)
+            {
+                info_lock.unlock();
+                known_statuses.push_back(info_to_details(info.second.get()));
+            }
+        }
+
+        // Invoke callback for all known service statuses
+        lock.unlock();
+        for (auto const& status : known_statuses)
+        {
+            callback_(status);
+        }
+    }
 }
 
 void OnlineAccountClientImpl::refresh_service_statuses()
@@ -340,6 +400,13 @@ void OnlineAccountClientImpl::refresh_service_statuses()
         std::rethrow_exception(thread_exception_);  // LCOV_EXCL_LINE
     }
 
+    // Update the accounts we already know about
+    for (auto const& info : accounts_)
+    {
+        service_update_cb(info.second->account_service.get(), ag_account_service_get_enabled(info.second->account_service.get()), info.second.get());
+    }
+
+    // Find new account we don't yet know about
     std::shared_ptr<GList> enabled_accounts(ag_manager_list(manager_.get()), ag_manager_list_free);
     GList* it;
     for (it = enabled_accounts.get(); it; it = it->next)
@@ -368,7 +435,7 @@ std::vector<OnlineAccountClient::ServiceStatus> OnlineAccountClientImpl::get_ser
     std::vector<OnlineAccountClient::ServiceStatus> service_statuses;
     for (auto const& info : accounts_)
     {
-        flush_pending_session(info.second->account_id, lock);
+        flush_pending_session(info.second.get(), lock);
         service_statuses.push_back(info_to_details(info.second.get()));
     }
     return service_statuses;
@@ -387,6 +454,7 @@ void OnlineAccountClientImpl::register_account_login_item(Result& result,
     }
 
     VariantMap account_details_map;
+    account_details_map["scope_id"] = query.scope_id();
     account_details_map["service_name"] = service_name_;
     account_details_map["service_type"] = service_type_;
     account_details_map["provider_name"] = provider_name_;
@@ -401,6 +469,7 @@ void OnlineAccountClientImpl::register_account_login_item(PreviewWidget& widget,
                                                           OnlineAccountClient::PostLoginAction login_failed_action)
 {
     VariantMap account_details_map;
+    account_details_map["scope_id"] = "";
     account_details_map["service_name"] = service_name_;
     account_details_map["service_type"] = service_type_;
     account_details_map["provider_name"] = provider_name_;
@@ -434,22 +503,19 @@ void OnlineAccountClientImpl::tear_down()
         for (auto const& info : accounts_)
         {
             // Before we nuke the map, ensure that any pending sessions are done
-            g_signal_handler_disconnect(info.second->account_service.get(), info.second->service_update_signal_id_);
-            flush_pending_session(info.second->account_id, lock);
+            {
+                std::lock_guard<std::mutex> info_lock(info.second->mutex);
+                g_signal_handler_disconnect(info.second->account_service.get(), info.second->service_update_signal_id_);
+            }
+            flush_pending_session(info.second.get(), lock);
         }
         accounts_.clear();
     }
 }
 
-void OnlineAccountClientImpl::flush_pending_session(AgAccountId const& account_id, std::unique_lock<std::mutex>& lock)
+void OnlineAccountClientImpl::flush_pending_session(AccountInfo* info, std::unique_lock<std::mutex>& lock)
 {
-    // Get account info
-    auto info_it = accounts_.find(account_id);
-    if (info_it == accounts_.end())
-    {
-        return;
-    }
-    auto info = info_it->second;
+    std::unique_lock<std::mutex> info_lock(info->mutex);
 
     // Wait until all currently running login sessions are done
     // (ensures that accounts_ is up to date)
@@ -464,9 +530,11 @@ void OnlineAccountClientImpl::flush_pending_session(AgAccountId const& account_i
         g_source_set_callback(source.get(), wake_up_event_loop_cb, event_loop.get(), NULL);
         g_source_attach(source.get(), main_loop_context_.get());
 
+        info_lock.unlock();
         lock.unlock();
         g_main_loop_run(event_loop.get());
         lock.lock();
+        info_lock.lock();
     }
 }
 
@@ -498,7 +566,6 @@ std::shared_ptr<AgManager> OnlineAccountClientImpl::manager()
 
 std::string OnlineAccountClientImpl::service_name()
 {
-    std::lock_guard<std::mutex> lock(mutex_);
     return service_name_;
 }
 
@@ -528,6 +595,7 @@ void OnlineAccountClientImpl::callback(AccountInfo* info, std::string const& err
     });
 
     // Clear session info
+    std::lock_guard<std::mutex> info_lock(info->mutex);
     info->session = nullptr;
 }
 
@@ -556,8 +624,11 @@ void OnlineAccountClientImpl::remove_account(AgAccountId const& account_id)
     auto info = info_it->second;
 
     // Before we nuke the pointer, ensure that any pending sessions are done
-    g_signal_handler_disconnect(info->account_service.get(), info->service_update_signal_id_);
-    flush_pending_session(account_id, lock);
+    {
+        std::lock_guard<std::mutex> info_lock(info->mutex);
+        g_signal_handler_disconnect(info->account_service.get(), info->service_update_signal_id_);
+    }
+    flush_pending_session(info.get(), lock);
 
     // Remove account info from accounts_ map
     accounts_.erase(account_id);
@@ -584,19 +655,19 @@ void OnlineAccountClientImpl::main_loop_thread()
     // LCOV_EXCL_START
     catch (std::exception const& e)
     {
-        std::cerr << "OnlineAccountClientImpl::main_loop_thread(): Thread aborted: " << e.what() << std::endl;
         std::lock_guard<std::mutex> lock(mutex_);
         thread_exception_ = std::current_exception();
     }
     catch (...)
     {
-        std::cerr << "OnlineAccountClientImpl::main_loop_thread(): Thread aborted: unknown exception" << std::endl;
         std::lock_guard<std::mutex> lock(mutex_);
         thread_exception_ = std::current_exception();
     }
     // LCOV_EXCL_STOP
 }
 
+#pragma GCC diagnostic pop
+
 }  // namespace internal
 
 }  // namespace scopes
