@@ -30,10 +30,8 @@
 #include <core/posix/child_process.h>
 #include <core/posix/exec.h>
 
-#include <cassert>
 #include <fstream>
 #include <wordexp.h>
-#include <iostream>
 
 using namespace std;
 
@@ -49,11 +47,15 @@ namespace scopes
 namespace internal
 {
 
+// Substitute logger for testing. (Some of tests mock out the middleware and run time.)
+BOOST_LOG_INLINE_GLOBAL_LOGGER_DEFAULT(registry_object_test_logger, boost::log::sources::severity_channel_logger_mt<>)
+
 RegistryObject::RegistryObject(core::posix::ChildProcess::DeathObserver& death_observer,
                                Executor::SPtr const& executor,
                                MiddlewareBase::SPtr middleware,
                                bool generate_desktop_files)
-    : death_observer_(death_observer),
+    : logger_(middleware ? middleware->runtime()->logger() : registry_object_test_logger::get()),
+      death_observer_(death_observer),
       death_observer_connection_
       {
           death_observer_.child_died().connect([this](core::posix::ChildProcess const& cp)
@@ -78,10 +80,19 @@ RegistryObject::RegistryObject(core::posix::ChildProcess::DeathObserver& death_o
         try
         {
             publisher_ = middleware->create_publisher(middleware->runtime()->registry_identity());
+
+            ss_list_update_subscriber_ = middleware->create_subscriber(middleware->runtime()->ss_registry_identity());
+            ss_list_update_connection_ = std::make_shared<core::ScopedConnection>
+            (
+                ss_list_update_subscriber_->message_received().connect([this](string const&)
+                {
+                    ss_list_update();
+                })
+            );
         }
         catch (std::exception const& e)
         {
-            std::cerr << "RegistryObject(): failed to create registry publisher: " << e.what() << endl;
+            BOOST_LOG(logger_) << "RegistryObject(): failed to create registry publisher: " << e.what();
         }
     }
 }
@@ -104,13 +115,13 @@ RegistryObject::~RegistryObject()
             // outputting bogus error messages.
             if (is_scope_running(scope_process.first))
             {
-                scope_process.second.update_state(ScopeProcess::Stopping);
-                scope_process.second.kill();
+                scope_process.second->update_state(ScopeProcess::Stopping);
+                scope_process.second->kill();
             }
         }
         catch(std::exception const& e)
         {
-            cerr << "RegistryObject::~RegistryObject(): " << e.what() << endl;
+            BOOST_LOG(logger_) << "RegistryObject::~RegistryObject(): " << e.what();
         }
     }
 }
@@ -144,8 +155,7 @@ ScopeMetadata RegistryObject::get_metadata(std::string const& scope_id) const
         }
         catch (std::exception const& e)
         {
-            cerr << "cannot get metadata from remote registry: " << e.what() << endl;
-            // TODO: log error
+            BOOST_LOG(logger_) << "cannot get metadata from remote registry: " << e.what();
         }
     }
 
@@ -173,8 +183,7 @@ MetadataMap RegistryObject::list() const
         }
         catch (std::exception const& e)
         {
-            cerr << "cannot get scopes list from remote registry: " << e.what() << endl;
-            // TODO: log error
+            BOOST_LOG(logger_) << "cannot get scopes list from remote registry: " << e.what();
         }
     }
 
@@ -190,7 +199,7 @@ ObjectProxy RegistryObject::locate(std::string const& identity)
     }
 
     ObjectProxy proxy;
-    ProcessMap::iterator proc_it;
+    shared_ptr<ScopeProcess> proc;
     {
         lock_guard<decltype(mutex_)> lock(mutex_);
 
@@ -201,15 +210,17 @@ ObjectProxy RegistryObject::locate(std::string const& identity)
         }
         proxy = scope_it->second.proxy();
 
-        proc_it = scope_processes_.find(identity);
+        auto proc_it = scope_processes_.find(identity);
         if (proc_it == scope_processes_.end())
         {
             throw NotFoundException("RegistryObject::locate(): Tried to exec unknown local scope", identity);
         }
+        proc = proc_it->second;
     }
 
     // Exec after unlocking, so we can start processing another locate()
-    proc_it->second.exec(death_observer_, executor_);
+    assert(proc);
+    proc->exec(death_observer_, executor_);
 
     return proxy;
 }
@@ -221,7 +232,7 @@ bool RegistryObject::is_scope_running(std::string const& scope_id)
     auto it = scope_processes_.find(scope_id);
     if (it != scope_processes_.end())
     {
-        return it->second.state() == ScopeProcess::ProcessState::Running;
+        return it->second->state() == ScopeProcess::ProcessState::Running;
     }
 
     throw NotFoundException("RegistryObject::is_scope_process_running(): no such scope: ",  scope_id);
@@ -244,12 +255,14 @@ bool RegistryObject::add_local_scope(std::string const& scope_id, ScopeMetadata
     bool return_value = true;
     if (scopes_.find(scope_id) != scopes_.end())
     {
+        // If scope is known already, remove it's details and kill it if it is running.
         scopes_.erase(scope_id);
         scope_processes_.erase(scope_id);
+        remove_desktop_file(scope_id);
         return_value = false;
     }
     scopes_.insert(make_pair(scope_id, metadata));
-    scope_processes_.insert(make_pair(scope_id, ScopeProcess(exec_data, publisher_)));
+    scope_processes_.insert(make_pair(scope_id, make_shared<ScopeProcess>(exec_data, publisher_, logger_)));
 
     if (publisher_)
     {
@@ -270,31 +283,55 @@ bool RegistryObject::remove_local_scope(std::string const& scope_id)
                                               "with empty id");
     }
 
-    unique_lock<decltype(mutex_)> lock(mutex_);
-
-    auto proc_it = scope_processes_.find(scope_id);
-    if (proc_it != scope_processes_.end())
+    shared_ptr<ScopeProcess> proc;
     {
-        // Kill process after unlocking, so we can handle on_process_death
-        lock.unlock();
-        proc_it->second.kill();
-        lock.lock();
-    }
+        unique_lock<decltype(mutex_)> lock(mutex_);
 
-    scope_processes_.erase(scope_id);
-
-    if (scopes_.erase(scope_id) == 1)
-    {
-        if (publisher_)
+        auto proc_it = scope_processes_.find(scope_id);
+        if (proc_it != scope_processes_.end())
         {
-            // Send a blank message to subscribers to inform them that the registry has been updated
-            publisher_->send_message("");
+            proc = proc_it->second;
         }
-        remove_desktop_file(scope_id);
-        return true;
     }
 
-    return false;
+    // Kill process after unlocking, so we can handle on_process_death
+    exception_ptr ex;
+    if (proc)
+    {
+        try
+        {
+            proc->kill();
+        }
+        catch (...)
+        {
+            ex = current_exception();
+        }
+    }
+
+    bool erased = false;
+    {
+        unique_lock<decltype(mutex_)> lock(mutex_);
+
+        scope_processes_.erase(scope_id);
+        erased = scopes_.erase(scope_id) == 1;
+        if (erased)
+        {
+            remove_desktop_file(scope_id);
+        }
+    }
+
+    if (publisher_ && erased)
+    {
+        // Send a blank message to subscribers to inform them that the registry has been updated
+        publisher_->send_message("");
+    }
+
+    if (ex)
+    {
+        rethrow_exception(ex);
+    }
+
+    return erased;
 }
 
 void RegistryObject::set_remote_registry(MWRegistryProxy const& remote_registry)
@@ -318,7 +355,7 @@ void RegistryObject::on_process_death(core::posix::ChildProcess const& process)
     pid_t pid = process.pid();
     for (auto& scope_process : scope_processes_)
     {
-        if (scope_process.second.on_process_death(pid))
+        if (scope_process.second->on_process_death(pid))
         {
             break;
         }
@@ -335,13 +372,14 @@ void RegistryObject::on_state_received(std::string const& scope_id, StateReceive
         switch (state)
         {
             case StateReceiverObject::ScopeReady:
-                it->second.update_state(ScopeProcess::ProcessState::Running);
+                it->second->update_state(ScopeProcess::ProcessState::Running);
                 break;
             case StateReceiverObject::ScopeStopping:
-                it->second.update_state(ScopeProcess::ProcessState::Stopping);
+                it->second->update_state(ScopeProcess::ProcessState::Stopping);
                 break;
             default:
-                std::cerr << "RegistryObject::on_state_received(): unknown state received from scope: " << scope_id;
+                BOOST_LOG(logger_)
+                    << "RegistryObject::on_state_received(): unknown state received from scope: " << scope_id;
         }
     }
     // simply ignore states from scopes the registry does not know about
@@ -415,15 +453,22 @@ void RegistryObject::remove_desktop_file(std::string const& scope_id)
     }
 }
 
-RegistryObject::ScopeProcess::ScopeProcess(ScopeExecData exec_data, std::weak_ptr<MWPublisher> const& publisher)
+void RegistryObject::ss_list_update()
+{
+    if (publisher_)
+    {
+        // Send a blank message to subscribers to inform them that the smart scopes proxy has been updated
+        publisher_->send_message("");
+    }
+}
+
+RegistryObject::ScopeProcess::ScopeProcess(ScopeExecData exec_data,
+                                           std::weak_ptr<MWPublisher> const& publisher,
+                                           boost::log::sources::severity_channel_logger_mt<>& logger)
     : exec_data_(exec_data)
     , reg_publisher_(publisher)
     , manually_started_(false)
-{
-}
-
-RegistryObject::ScopeProcess::ScopeProcess(ScopeProcess const& other)
-    : ScopeProcess(other.exec_data_, other.reg_publisher_)
+    , logger_(logger)
 {
 }
 
@@ -435,7 +480,7 @@ RegistryObject::ScopeProcess::~ScopeProcess()
     }
     catch(std::exception const& e)
     {
-        cerr << "RegistryObject::ScopeProcess::~ScopeProcess(): " << e.what() << endl;
+        BOOST_LOG(logger_) << "RegistryObject::ScopeProcess::~ScopeProcess(): " << e.what();
     }
 }
 
@@ -474,15 +519,16 @@ void RegistryObject::ScopeProcess::exec(
     {
         if (!wait_for_state(lock, ScopeProcess::Stopped))
         {
-            cerr << "RegistryObject::ScopeProcess::exec(): Force killing process. Scope: \""
-                 << exec_data_.scope_id << "\" did not stop after " << exec_data_.timeout_ms << " ms." << endl;
+            BOOST_LOG(logger_)
+                << "RegistryObject::ScopeProcess::exec(): Force killing process. Scope: \""
+                << exec_data_.scope_id << "\" did not stop after " << exec_data_.timeout_ms << " ms.";
             try
             {
                 kill(lock);
             }
             catch(std::exception const& e)
             {
-                cerr << "RegistryObject::ScopeProcess::exec(): kill() failed: " << e.what() << endl;
+                BOOST_LOG(logger_) << "RegistryObject::ScopeProcess::exec(): kill() failed: " << e.what();
             }
         }
     }
@@ -523,6 +569,7 @@ void RegistryObject::ScopeProcess::exec(
         auto scope_config_path = boost::filesystem::canonical(exec_data_.scope_config);
         string lib_dir = scope_config_path.parent_path().native();
         string scope_ld_lib_path = lib_dir + ":" + lib_dir + "/lib";
+        scope_ld_lib_path.append(":" + lib_dir + "/" + DEB_HOST_MULTIARCH + "/lib");
         string ld_lib_path = core::posix::this_process::env::get("LD_LIBRARY_PATH", "");
         if (!boost::algorithm::starts_with(ld_lib_path, lib_dir))
         {
@@ -531,7 +578,7 @@ void RegistryObject::ScopeProcess::exec(
         env["LD_LIBRARY_PATH"] = scope_ld_lib_path;  // Overwrite any LD_LIBRARY_PATH entry that may already be there.
 
         process_ = executor->exec(program, argv, env,
-                                  core::posix::StandardStream::stdin | core::posix::StandardStream::stdout,
+                                  core::posix::StandardStream::stdin,
                                   exec_data_.confinement_profile);
         if (process_.pid() <= 0)
         {
@@ -553,14 +600,15 @@ void RegistryObject::ScopeProcess::exec(
         }
         catch(std::exception const& e)
         {
-            cerr << "RegistryObject::ScopeProcess::exec(): kill() failed: " << e.what() << endl;
+            BOOST_LOG(logger_) << "RegistryObject::ScopeProcess::exec(): kill() failed: " << e.what();
         }
         throw unity::ResourceException("RegistryObject::ScopeProcess::exec(): exec aborted. Scope: \""
                                        + exec_data_.scope_id + "\" took longer than "
                                        + std::to_string(exec_data_.timeout_ms) + " ms to start.");
     }
 
-    cout << "RegistryObject::ScopeProcess::exec(): Process for scope: \"" << exec_data_.scope_id << "\" started" << endl;
+    BOOST_LOG_SEV(logger_, Logger::Info)
+        << "RegistryObject::ScopeProcess::exec(): Process for scope: \"" << exec_data_.scope_id << "\" started";
 
     // 4. add the scope process to the death observer
     death_observer.add(process_);
@@ -579,8 +627,9 @@ bool RegistryObject::ScopeProcess::on_process_death(pid_t pid)
     // check if this is the process reported to have died
     if (pid == process_.pid())
     {
-        cout << "RegistryObject::ScopeProcess::on_process_death(): Process for scope: \"" << exec_data_.scope_id
-             << "\" exited" << endl;
+        BOOST_LOG_SEV(logger_, Logger::Info)
+            << "RegistryObject::ScopeProcess::on_process_death(): Process for scope: \"" << exec_data_.scope_id
+            << "\" exited";
         clear_handle_unlocked();
         return true;
     }
@@ -611,8 +660,9 @@ void RegistryObject::ScopeProcess::update_state_unlocked(ProcessState new_state)
 
         if (state_ != Starting)
         {
-            cout << "RegistryObject::ScopeProcess: Process for scope: \"" << exec_data_.scope_id
-                 << "\" started manually" << endl;
+            BOOST_LOG_SEV(logger_, Logger::Info)
+                << "RegistryObject::ScopeProcess: Process for scope: \"" << exec_data_.scope_id
+                << "\" started manually";
 
             manually_started_ = true;
         }
@@ -627,8 +677,9 @@ void RegistryObject::ScopeProcess::update_state_unlocked(ProcessState new_state)
 
         if (state_ != Stopping)
         {
-            cerr << "RegistryObject::ScopeProcess: Scope: \"" << exec_data_.scope_id
-                 << "\" closed unexpectedly. Either the process crashed or was killed forcefully." << endl;
+            BOOST_LOG(logger_)
+                << "RegistryObject::ScopeProcess: Scope: \"" << exec_data_.scope_id
+                << "\" closed unexpectedly. Either the process crashed or was killed forcefully.";
         }
     }
     else if (new_state == Stopping && manually_started_)
@@ -639,8 +690,9 @@ void RegistryObject::ScopeProcess::update_state_unlocked(ProcessState new_state)
             reg_publisher->send_message("stopped", exec_data_.scope_id);
         }
 
-        cout << "RegistryObject::ScopeProcess: Manually started process for scope: \""
-             << exec_data_.scope_id << "\" exited" << endl;
+        BOOST_LOG(logger_)
+            << "RegistryObject::ScopeProcess: Manually started process for scope: \""
+            << exec_data_.scope_id << "\" exited";
 
         new_state = Stopped;
         manually_started_ = false;
@@ -672,9 +724,10 @@ void RegistryObject::ScopeProcess::kill(std::unique_lock<std::mutex>& lock)
         {
             std::error_code ec;
 
-            cerr << "RegistryObject::ScopeProcess::kill(): Scope: \"" << exec_data_.scope_id
-                 << "\" took longer than " << exec_data_.timeout_ms << " ms to exit gracefully. "
-                 << "Killing the process instead." << endl;
+            BOOST_LOG(logger_)
+                << "RegistryObject::ScopeProcess::kill(): Scope: \"" << exec_data_.scope_id
+                << "\" took longer than " << exec_data_.timeout_ms << " ms to exit gracefully. "
+                << "Killing the process instead.";
 
             // scope is taking too long to close, send kill signal
             process_.send_signal(core::posix::Signal::sig_kill, ec);
@@ -685,8 +738,8 @@ void RegistryObject::ScopeProcess::kill(std::unique_lock<std::mutex>& lock)
     }
     catch (std::exception const&)
     {
-        cerr << "RegistryObject::ScopeProcess::kill(): Failed to kill scope: \""
-             << exec_data_.scope_id << "\"" << endl;
+        BOOST_LOG(logger_)
+            << "RegistryObject::ScopeProcess::kill(): Failed to kill scope: \"" << exec_data_.scope_id << "\"";
 
         // clear the process handle
         // even on error, the previous handle will be unrecoverable at this point
@@ -713,7 +766,7 @@ std::vector<std::string> RegistryObject::ScopeProcess::expand_custom_exec()
         util::ResourcePtr<wordexp_t*, decltype(&wordfree)> free_guard(&exp, wordfree);
 
         command_args.push_back(exp.we_wordv[0]);
-        for (uint i = 1; i < exp.we_wordc; ++i)
+        for (unsigned int i = 1; i < exp.we_wordc; ++i)
         {
             std::string arg = exp.we_wordv[i];
             // Replace "%R" placeholders with the runtime config
@@ -761,9 +814,10 @@ void RegistryObject::ScopeProcess::publish_state_change(ProcessState scope_state
             started_message += " string:" + exec_data_.scope_id + " uint64:" + std::to_string(process_.pid());
             if (safe_system_call(started_message) != 0)
             {
-                std::cerr << "RegistryObject::ScopeProcess::publish_state_change(): "
-                             "Failed to execute SDK DBus ScopeLoaded callback "
-                             "(Scope ID: " << exec_data_.scope_id << ")" << endl;
+                BOOST_LOG(logger_)
+                    << "RegistryObject::ScopeProcess::publish_state_change(): "
+                       "Failed to execute SDK DBus ScopeLoaded callback "
+                       "(Scope ID: " << exec_data_.scope_id << ")";
             }
         }
     }
@@ -781,9 +835,10 @@ void RegistryObject::ScopeProcess::publish_state_change(ProcessState scope_state
             stopped_message += " string:" + exec_data_.scope_id;
             if (safe_system_call(stopped_message) != 0)
             {
-                std::cerr << "RegistryObject::ScopeProcess::publish_state_change(): "
-                             "Failed to execute SDK DBus ScopeStopped callback "
-                             "(Scope ID: " << exec_data_.scope_id << ")" << endl;
+                BOOST_LOG(logger_)
+                    << "RegistryObject::ScopeProcess::publish_state_change(): "
+                       "Failed to execute SDK DBus ScopeStopped callback "
+                       "(Scope ID: " << exec_data_.scope_id << ")";
             }
         }
     }
