@@ -26,7 +26,6 @@
 #include <unity/UnityExceptions.h>
 
 #include <cassert>
-#include <iostream> // TODO: remove this once logging is added
 
 using namespace std;
 
@@ -39,10 +38,10 @@ namespace scopes
 namespace internal
 {
 
-ReplyImpl::ReplyImpl(MWReplyProxy const& mw_proxy, std::shared_ptr<QueryObjectBase> const& qo) :
-    ObjectImpl(mw_proxy),
-    qo_(qo),
-    finished_(false)
+ReplyImpl::ReplyImpl(MWReplyProxy const& mw_proxy, std::shared_ptr<QueryObjectBase> const& qo)
+    : ObjectImpl(mw_proxy)
+    , qo_(qo)
+    , finished_(false)
 {
     assert(mw_proxy);
 }
@@ -55,7 +54,6 @@ ReplyImpl::~ReplyImpl()
     }
     catch (...)
     {
-        // TODO: log error
     }
 }
 
@@ -94,10 +92,9 @@ void ReplyImpl::finished()
         {
             fwd()->finished(CompletionDetails(CompletionDetails::OK));  // Oneway, can't block
         }
-        catch (std::exception const& e)
+        catch (std::exception const&)
         {
-            // TODO: log error
-            cerr << e.what() << endl;
+            // No logging here because this may happen after the run time is destroyed.
         }
     }
 }
@@ -124,8 +121,7 @@ void ReplyImpl::error(exception_ptr ex)
     {
         error_message = "unknown exception";
     }
-    // TODO: log error
-    cerr << error_message << endl;
+    BOOST_LOG(mw_proxy_->mw_base()->runtime()->logger()) << "ReplyImpl::error(): " << error_message;
 
     try
     {
@@ -133,8 +129,8 @@ void ReplyImpl::error(exception_ptr ex)
     }
     catch (std::exception const& e)
     {
-        // TODO: log error
-        cerr << e.what() << endl;
+        BOOST_LOG(mw_proxy_->mw_base()->runtime()->logger())
+            << "ReplyImpl::error(): exception from finished(): " << error_message;
     }
 }
 
@@ -151,8 +147,8 @@ void ReplyImpl::info(OperationInfo const& op_info)
     }
     catch (std::exception const& e)
     {
-        // TODO: log error
-        cerr << e.what() << endl;
+        BOOST_LOG(mw_proxy_->mw_base()->runtime()->logger())
+            << "ReplyImpl::error(): exception from info(): " << e.what();
     }
 }
 
