@@ -19,8 +19,16 @@
 #include <unity/scopes/internal/RuntimeConfig.h>
 
 #include <unity/scopes/internal/DfltConfig.h>
+#include <unity/scopes/ScopeExceptions.h>
+
+#include <boost/algorithm/string/classification.hpp>
+#include <boost/algorithm/string/constants.hpp>
+#include <boost/algorithm/string/split.hpp>
+#include <boost/filesystem.hpp>
 #include <unity/UnityExceptions.h>
 
+#include <sys/stat.h>
+
 #include <stdlib.h>
 
 using namespace std;
@@ -46,8 +54,13 @@ const string default_middleware_key = "Default.Middleware";
 const string default_middleware_configfile_key = ".ConfigFile";
 const string reap_expiry_key = "Reap.Expiry";
 const string reap_interval_key = "Reap.Interval";
-const string data_dir_key = "DataDir";
+const string cache_dir_key = "CacheDir";
+const string app_dir_key = "AppDir";
 const string config_dir_key = "ConfigDir";
+const string log_dir_key = "LogDir";
+const string max_log_file_size_key = "Log.MaxFileSize";
+const string max_log_dir_size_key = "Log.MaxDirSize";
+const string trace_channels_key = "Log.TraceChannels";
 
 }  // namespace
 
@@ -64,15 +77,12 @@ RuntimeConfig::RuntimeConfig(string const& configfile) :
         default_middleware_configfile_ = DFLT_ZMQ_MIDDLEWARE_INI;
         reap_expiry_ = DFLT_REAP_EXPIRY;
         reap_interval_ = DFLT_REAP_INTERVAL;
-        try
-        {
-            data_directory_ = default_data_directory();
-            config_directory_ = default_config_directory();
-        }
-        catch (ResourceException const& e)
-        {
-            throw_ex("Failed to get default data directory");
-        }
+        max_log_file_size_ = DFLT_MAX_LOG_FILE_SIZE;
+        max_log_dir_size_ = DFLT_MAX_LOG_DIR_SIZE;
+        cache_directory_ = default_cache_directory();
+        app_directory_ = default_app_directory();
+        config_directory_ = default_config_directory();
+        log_directory_ = default_log_directory();
     }
     else
     {
@@ -94,18 +104,33 @@ RuntimeConfig::RuntimeConfig(string const& configfile) :
         {
             throw_ex("Illegal value (" + to_string(reap_interval_) + ") for " + reap_interval_key + ": value must be > 0");
         }
-        data_directory_ = get_optional_string(runtime_config_group, data_dir_key);
-        if (data_directory_.empty())
+
+        cache_directory_ = get_optional_string(runtime_config_group, cache_dir_key);
+        if (cache_directory_.empty())
         {
             try
             {
-                data_directory_ = default_data_directory();
+                cache_directory_ = default_cache_directory();
             }
             catch (ResourceException const& e)
             {
-                throw_ex("No DataDir configured and failed to get default");
+                throw_ex("No " + cache_dir_key + " configured and failed to get default");
             }
         }
+
+        app_directory_ = get_optional_string(runtime_config_group, app_dir_key);
+        if (app_directory_.empty())
+        {
+            try
+            {
+                app_directory_ = default_app_directory();
+            }
+            catch (ResourceException const& e)
+            {
+                throw_ex("No " + app_dir_key + " configured and failed to get default");
+            }
+        }
+
         config_directory_ = get_optional_string(runtime_config_group, config_dir_key);
         if (config_directory_.empty())
         {
@@ -115,9 +140,66 @@ RuntimeConfig::RuntimeConfig(string const& configfile) :
             }
             catch (ResourceException const& e)
             {
-                throw_ex("No ConfigDir configured and failed to get default");
+                throw_ex("No " + config_dir_key + " configured and failed to get default");
             }
         }
+
+        try
+        {
+            // If explicitly set to the empty string, we succeed here.
+            log_directory_ = parser()->get_string(runtime_config_group, log_dir_key);
+        }
+        catch (LogicException const&)
+        {
+            // Use default value.
+            try
+            {
+                log_directory_ = default_log_directory();
+            }
+            catch (ResourceException const& e)
+            {
+                throw_ex("No " + log_dir_key + " configured and failed to get default");
+            }
+        }
+
+        // Check if we have an override for the log directory.
+        char const* logdir = getenv("UNITY_SCOPES_LOGDIR");
+        if (logdir)
+        {
+            log_directory_ = logdir;
+        }
+
+        max_log_file_size_ = get_optional_int(runtime_config_group, max_log_file_size_key, DFLT_MAX_LOG_FILE_SIZE);
+        if (max_log_file_size_ < 1024)
+        {
+            throw_ex("Illegal value (" + to_string(max_log_file_size_) + ") for " + max_log_file_size_key
+                     + ": value must be > 1024");
+        }
+
+        max_log_dir_size_ = get_optional_int(runtime_config_group, max_log_dir_size_key, DFLT_MAX_LOG_DIR_SIZE);
+        if (max_log_dir_size_ <= max_log_file_size_)
+        {
+            throw_ex("Illegal value (" + to_string(max_log_dir_size_) + ") for " + max_log_dir_size_key
+                     + ": value must be > " + max_log_file_size_key + " (" + to_string(max_log_file_size_) + ")");
+        }
+
+        try
+        {
+            trace_channels_ = parser()->get_string_array(runtime_config_group, trace_channels_key);
+        }
+        catch (LogicException const&)
+        {
+            // No TraceChannels configured.
+        }
+
+        // Check if we have an override for the trace channels.
+        char const* tc = getenv("UNITY_SCOPES_LOG_TRACECHANNELS");
+        if (tc && *tc != '\0')
+        {
+            vector<string> channels;
+            split(channels, tc, boost::is_any_of(";"), boost::token_compress_on);
+            trace_channels_ = channels;
+        }
     }
 
     KnownEntries const known_entries = {
@@ -131,8 +213,13 @@ RuntimeConfig::RuntimeConfig(string const& configfile) :
                                                 default_middleware_ + default_middleware_configfile_key,
                                                 reap_expiry_key,
                                                 reap_interval_key,
-                                                data_dir_key,
-                                                config_dir_key
+                                                cache_dir_key,
+                                                app_dir_key,
+                                                config_dir_key,
+                                                log_dir_key,
+                                                max_log_file_size_key,
+                                                max_log_dir_size_key,
+                                                trace_channels_key
                                              }
                                           }
                                        };
@@ -183,9 +270,14 @@ int RuntimeConfig::reap_interval() const
     return reap_interval_;
 }
 
-string RuntimeConfig::data_directory() const
+string RuntimeConfig::cache_directory() const
 {
-    return data_directory_;
+    return cache_directory_;
+}
+
+string RuntimeConfig::app_directory() const
+{
+    return app_directory_;
 }
 
 string RuntimeConfig::config_directory() const
@@ -193,6 +285,46 @@ string RuntimeConfig::config_directory() const
     return config_directory_;
 }
 
+string RuntimeConfig::log_directory() const
+{
+    return log_directory_;
+}
+
+int RuntimeConfig::max_log_file_size() const
+{
+    return max_log_file_size_;
+}
+
+int RuntimeConfig::max_log_dir_size() const
+{
+    return max_log_dir_size_;
+}
+
+vector<string> RuntimeConfig::trace_channels() const
+{
+    return trace_channels_;
+}
+
+string RuntimeConfig::default_cache_directory()
+{
+    char const* home = getenv("HOME");
+    if (!home || *home == '\0')
+    {
+        throw ResourceException("RuntimeConfig::default_cache_directory(): $HOME not set");
+    }
+    return string(home) + "/" + DFLT_HOME_CACHE_SUBDIR;
+}
+
+string RuntimeConfig::default_app_directory()
+{
+    char const* home = getenv("HOME");
+    if (!home || *home == '\0')
+    {
+        throw ResourceException("RuntimeConfig::default_app_directory(): $HOME not set");
+    }
+    return string(home) + "/" + DFLT_HOME_APP_SUBDIR;
+}
+
 string RuntimeConfig::default_config_directory()
 {
     char const* home = getenv("HOME");
@@ -200,19 +332,23 @@ string RuntimeConfig::default_config_directory()
     {
         throw ResourceException("RuntimeConfig::default_config_directory(): $HOME not set");
     }
-    return string(home) + "/.config/unity-scopes";
+    return string(home) + "/" + DFLT_HOME_CONFIG_SUBDIR;
 }
 
-string RuntimeConfig::default_data_directory()
+string RuntimeConfig::default_log_directory()
 {
     char const* home = getenv("HOME");
     if (!home || *home == '\0')
     {
-        throw ResourceException("RuntimeConfig::default_data_directory(): $HOME not set");
+        throw ResourceException("RuntimeConfig::default_log_directory(): $HOME not set");
     }
-    return string(home) + "/.local/share/unity-scopes";
-}
+    string dir = string(home) + "/" + DFLT_HOME_LOG_SUBDIR;
 
+    boost::system::error_code ec;
+    !boost::filesystem::exists(dir, ec) && ::mkdir(dir.c_str(), 0700);
+
+    return dir;
+}
 
 } // namespace internal
 
