@@ -17,28 +17,29 @@
  */
 
 #include <unity/scopes/internal/RuntimeImpl.h>
-#include <unity/scopes/internal/ScopeBaseImpl.h>
 
+#include <unity/scopes/internal/ChildScopesRepository.h>
 #include <unity/scopes/internal/DfltConfig.h>
+#include <unity/scopes/internal/Logger.h>
 #include <unity/scopes/internal/MWStateReceiver.h>
 #include <unity/scopes/internal/RegistryConfig.h>
 #include <unity/scopes/internal/RegistryImpl.h>
 #include <unity/scopes/internal/RuntimeConfig.h>
+#include <unity/scopes/internal/ScopeBaseImpl.h>
 #include <unity/scopes/internal/ScopeConfig.h>
 #include <unity/scopes/internal/ScopeObject.h>
 #include <unity/scopes/internal/SettingsDB.h>
 #include <unity/scopes/internal/UniqueID.h>
 #include <unity/scopes/ScopeBase.h>
 #include <unity/scopes/ScopeExceptions.h>
-#include <unity/UnityExceptions.h>
-#include <unity/util/FileIO.h>
 
 #include <boost/filesystem.hpp>
+#include <unity/UnityExceptions.h>
+#include <unity/util/FileIO.h>
 
 #include <cassert>
 #include <cstring>
 #include <future>
-#include <iostream> // TODO: remove this once logging is added
 
 #include <sys/apparmor.h>
 #include <sys/stat.h>
@@ -59,19 +60,48 @@ RuntimeImpl::RuntimeImpl(string const& scope_id, string const& configfile)
 {
     try
     {
-        lock_guard<mutex> lock(mutex_);
+        lock_guard<mutex> lock(mutex_);  // TODO: shouldn't be necessary.
 
         destroyed_ = false;
 
         scope_id_ = scope_id;
+        string log_file_basename = scope_id;
         if (scope_id_.empty())
         {
             UniqueID id;
             scope_id_ = "c-" + id.gen();
+            log_file_basename = "client";  // Don't make lots of log files named like c-c4c758b100000000.
         }
 
+        // Until we know where the scope's cache directory is,
+        // we use a logger that logs to std::clog.
+        logger_.reset(new Logger(scope_id_));
+
         // Create the middleware factory and get the registry identity and config filename.
+        runtime_configfile_ = configfile;
         RuntimeConfig config(configfile);
+
+        // Configure trace channels.
+        try
+        {
+            logger_->enable_channels(config.trace_channels());
+        }
+        catch (...)
+        {
+            throw InvalidArgumentException("Runtime(): invalid trace channel name(s)");
+        }
+
+        // Now that we have the config, change the logger to log to a file.
+        // If log_dir was explicitly set to the empty string, continue
+        // logging to std::clog.
+        log_dir_ = config.log_directory();
+        if (!log_dir_.empty())
+        {
+            logger_->set_log_file(find_log_dir(log_file_basename) + "/" + log_file_basename,
+                                  config.max_log_file_size(),
+                                  config.max_log_dir_size());
+        }
+
         string default_middleware = config.default_middleware();
         string middleware_configfile = config.default_middleware_configfile();
         middleware_factory_.reset(new MiddlewareFactory(this));
@@ -91,7 +121,7 @@ RuntimeImpl::RuntimeImpl(string const& scope_id, string const& configfile)
 
         if (registry_configfile_.empty() || registry_identity_.empty())
         {
-            cerr << "Warning: no registry configured" << endl;
+            BOOST_LOG_SEV(logger(), Logger::Warning) << "no registry configured";
             registry_identity_ = "";
         }
         else
@@ -99,10 +129,11 @@ RuntimeImpl::RuntimeImpl(string const& scope_id, string const& configfile)
             // Create the registry proxy
             RegistryConfig reg_config(registry_identity_, registry_configfile_);
             auto registry_mw_proxy = middleware_->registry_proxy();
-            registry_ = make_shared<RegistryImpl>(registry_mw_proxy, this);
+            registry_ = make_shared<RegistryImpl>(registry_mw_proxy);
         }
 
-        data_dir_ = config.data_directory();
+        cache_dir_ = config.cache_directory();
+        app_dir_ = config.app_directory();
         config_dir_ = config.config_directory();
     }
     catch (unity::Exception const& e)
@@ -110,7 +141,9 @@ RuntimeImpl::RuntimeImpl(string const& scope_id, string const& configfile)
         destroy();
         string msg = "Cannot instantiate run time for " + (scope_id.empty() ? "client" : scope_id) +
                      ", config file: " + configfile;
-        throw ConfigException(msg);
+        ConfigException ex(msg);
+        BOOST_LOG(logger()) << ex.what();
+        throw ex;
     }
 }
 
@@ -122,13 +155,11 @@ RuntimeImpl::~RuntimeImpl()
     }
     catch (std::exception const& e) // LCOV_EXCL_LINE
     {
-        cerr << "~RuntimeImpl(): " << e.what() << endl;
-        // TODO: log error
+        BOOST_LOG(logger()) << "~RuntimeImpl(): " << e.what();
     }
     catch (...) // LCOV_EXCL_LINE
     {
-        cerr << "~RuntimeImpl(): unknown exception" << endl;
-        // TODO: log error
+        BOOST_LOG(logger()) << "~RuntimeImpl(): unknown exception";
     }
 }
 
@@ -282,6 +313,16 @@ ThreadSafeQueue<future<void>>::SPtr RuntimeImpl::future_queue() const
     return future_queue_;  // Immutable
 }
 
+boost::log::sources::severity_channel_logger_mt<>& RuntimeImpl::logger() const
+{
+    return *logger_;
+}
+
+boost::log::sources::severity_channel_logger_mt<>& RuntimeImpl::logger(Logger::Channel channel) const
+{
+    return (*logger_)(channel);
+}
+
 namespace
 {
 
@@ -318,78 +359,50 @@ private:
 void RuntimeImpl::run_scope(ScopeBase* scope_base,
                             string const& scope_ini_file,
                             std::promise<void> ready_promise)
-{
-    run_scope(scope_base, "", scope_ini_file, move(ready_promise));
-}
-
-void RuntimeImpl::run_scope(ScopeBase* scope_base,
-                            string const& runtime_ini_file,
-                            string const& scope_ini_file,
-                            std::promise<void> ready_promise)
 {
     if (!scope_base)
     {
         throw InvalidArgumentException("Runtime::run_scope(): scope_base cannot be nullptr");
     }
 
-    // Retrieve the registry middleware and create a proxy to its state receiver
+    // Create a middleware for this scope.
     RegistryConfig reg_conf(registry_identity_, registry_configfile_);
-    auto reg_runtime = create(registry_identity_, runtime_ini_file);
-    auto reg_mw = reg_runtime->factory()->find(registry_identity_, reg_conf.mw_kind());
-    auto reg_state_receiver = reg_mw->create_state_receiver_proxy("StateReceiver");
-
     auto mw = factory()->create(scope_id_, reg_conf.mw_kind(), reg_conf.mw_configfile());
 
-    {
-        boost::filesystem::path dir = boost::filesystem::canonical(scope_ini_file).parent_path();
-        scope_base->p->set_scope_directory(dir.native());
-    }
+    boost::filesystem::path scope_dir = boost::filesystem::canonical(scope_ini_file).parent_path();
+    scope_base->p->set_scope_directory(scope_dir.native());
 
     {
         // Try to open the scope settings database, if any.
         string config_dir = config_dir_ + "/" + scope_id_;
         string settings_db = config_dir + "/settings.ini";
 
-        string scope_data_dir = data_dir_ + "/" + scope_id_;
-        string scope_dir = scope_base->scope_directory();
+        string settings_schema = scope_dir.native() + "/" + scope_id_ + "-settings.ini";
 
-        string settings_schema = scope_dir + "/" + scope_id_ + "-settings.ini";
         boost::system::error_code ec;
         if (boost::filesystem::exists(settings_schema, ec))
         {
-            // Make sure the data directories exist. (No permission for group and others; data might be sensitive.)
-            !boost::filesystem::exists(data_dir_, ec) && ::mkdir(data_dir_.c_str(), 0700);
-            !boost::filesystem::exists(scope_data_dir, ec) && ::mkdir(scope_data_dir.c_str(), 0700);
-
-            shared_ptr<SettingsDB> db(SettingsDB::create_from_ini_file(settings_db, settings_schema));
+            shared_ptr<SettingsDB> db(SettingsDB::create_from_ini_file(settings_db, settings_schema, logger()));
             scope_base->p->set_settings_db(db);
         }
         else
         {
             scope_base->p->set_settings_db(nullptr);
         }
+
+        // Configure the child scopes repository
+        string child_scopes_repo = config_dir + "/child-scopes.json";
+
+        auto repo = make_shared<ChildScopesRepository>(child_scopes_repo, logger());
+        scope_base->p->set_child_scopes_repo(repo);
     }
 
     scope_base->p->set_registry(registry_);
+    scope_base->p->set_cache_directory(find_cache_dir());
+    scope_base->p->set_app_directory(find_app_dir());
+    scope_base->p->set_tmp_directory(find_tmp_dir());
 
-    string confinement_type;
-    scope_base->p->set_cache_directory(find_cache_dir(confinement_type));
-
-    {
-        // Set tmp dir.
-        // We need to create any directories under /run/user/<uid> because they might not
-        // exist. (/run/user/<uid> gets cleaned out periodically.)
-        string path = string("/run/user/") + std::to_string(geteuid());
-        path += "/scopes";
-        boost::system::error_code ec;
-        !boost::filesystem::exists(path, ec) && ::mkdir(path.c_str(), 0700);
-        path += "/" + confinement_type;
-        !boost::filesystem::exists(path, ec) && ::mkdir(path.c_str(), 0700);
-        path += "/" + scope_id_;
-        !boost::filesystem::exists(path, ec) && ::mkdir(path.c_str(), 0700);
-
-        scope_base->p->set_tmp_directory(path);
-    }
+    // TODO: redirect log messages to scope-specific file.
 
     try
     {
@@ -434,26 +447,34 @@ void RuntimeImpl::run_scope(ScopeBase* scope_base,
             // Check if this scope has requested debug mode, if so, disable the idle timeout
             ScopeConfig scope_config(scope_ini_file);
             int idle_timeout_ms = scope_config.debug_mode() ? -1 : scope_config.idle_timeout() * 1000;
-            auto scope = unique_ptr<internal::ScopeObject>(new internal::ScopeObject(this,
-                                                                                     scope_base,
+            auto scope = unique_ptr<internal::ScopeObject>(new internal::ScopeObject(scope_base,
                                                                                      scope_config.debug_mode()));
             mw->add_scope_object(scope_id_, move(scope), idle_timeout_ms);
         }
         else
         {
-            auto scope = unique_ptr<internal::ScopeObject>(new internal::ScopeObject(this, scope_base));
+            auto scope = unique_ptr<internal::ScopeObject>(new internal::ScopeObject(scope_base));
             mw->add_scope_object(scope_id_, move(scope));
         }
 
         // Inform the registry that this scope is now ready to process requests
-        reg_state_receiver->push_state(scope_id_, StateReceiverObject::State::ScopeReady);
+        {
+            auto reg_state_receiver = mw->create_registry_state_receiver_proxy("StateReceiver");
+            reg_state_receiver->push_state(scope_id_, StateReceiverObject::State::ScopeReady);
+        }
 
         promise.set_value();
         mw->wait_for_shutdown();
-        cleanup_scope.dealloc();   // Causes ScopeBase::run() to terminate if the scope is properly written
+        cleanup_scope.dealloc();   // Causes ScopeBase::run() to return if the scope is properly written
 
-        // Inform the registry that this scope is shutting down
-        reg_state_receiver->push_state(scope_id_, StateReceiverObject::State::ScopeStopping);
+        {
+            // mw is now shut down, so we need a separate one to send the state update to the registry.
+            auto sd_runtime = create(scope_id_ + "-shutdown", runtime_configfile_);
+            auto sd_mw = sd_runtime->factory()->find(scope_id_ + "-shutdown", reg_conf.mw_kind());
+            auto reg_state_receiver = sd_mw->create_registry_state_receiver_proxy("StateReceiver");
+            // Inform the registry that this scope is shutting down
+            reg_state_receiver->push_state(scope_id_, StateReceiverObject::State::ScopeStopping);
+        }
     }
     catch (...)
     {
@@ -508,23 +529,30 @@ string RuntimeImpl::proxy_to_string(ObjectProxy const& proxy) const
     }
 }
 
-string RuntimeImpl::find_cache_dir(string& confinement_type) const
+string RuntimeImpl::cache_directory() const
 {
-    // Find out whether we are confined. aa_getcon() returns -1 in that case.
-    char* con = nullptr;
-    char* mode;
-    int rc = aa_getcon(&con, &mode);
-    // Only con (not mode) must be deallocated
-    free(con);
-    confinement_type = rc == -1 ? "leaf-net" : "unconfined";
+    return find_cache_dir();
+}
 
+string RuntimeImpl::tmp_directory() const
+{
+    return find_tmp_dir();
+}
+
+string RuntimeImpl::config_directory() const
+{
+    return config_dir_;
+}
+
+string RuntimeImpl::demangled_id(string const& scope_id) const
+{
     // For scopes that are in a click package together with an app,
-    // such as YouTube, the cache directory is shared between the app and
-    // the scope. The cache directory name is the scope ID up to the first
+    // such as YouTube, the scope id is <scope_id>_<app_id>
+    // The app directory and cache directory names are the scope ID up to the first
     // underscore. For example, com.ubuntu.scopes.youtube_youtube is the
     // scope ID, but the cache dir name is com.ubuntu.scopes.youtube.
-    auto id = scope_id_;
-    if (confinement_type == "leaf-net")
+    auto id = scope_id;
+    if (confined())
     {
         auto pos = id.find('_');
         if (pos != string::npos)
@@ -532,14 +560,94 @@ string RuntimeImpl::find_cache_dir(string& confinement_type) const
             id = id.substr(0, pos);
         }
     }
+    return id;
+}
 
-    // Create the data_dir_/<confinement-type>/<id> directories if they don't exist.
+bool RuntimeImpl::confined() const
+{
+    auto is_confined = []
+    {
+        // Find out whether we are confined. aa_getcon() returns -1 with EACCESS in that case.
+        char* con = nullptr;
+        char* mode;
+        int rc = aa_getcon(&con, &mode);
+        int error = errno;
+        if (rc == -1)
+        {
+            cerr << "aa_getcon failed, errno = " << error << endl;
+        }
+        // Only con (not mode) must be deallocated
+        free(con);
+        return rc == -1 && error == EACCES;
+    };
+    static bool confined = is_confined();
+    return confined;
+}
+
+string RuntimeImpl::confinement_type() const
+{
+    return confined() ? "leaf-net" : "unconfined";
+}
+
+string RuntimeImpl::find_cache_dir() const
+{
+    // Create the cache_dir_/<confinement-type>/<id> directories if they don't exist.
     boost::system::error_code ec;
-    !boost::filesystem::exists(data_dir_, ec) && ::mkdir(data_dir_.c_str(), 0700);
-    string dir = data_dir_ + "/" + confinement_type;
+    !confined() && !boost::filesystem::exists(cache_dir_, ec) && ::mkdir(cache_dir_.c_str(), 0700);
+    string dir = cache_dir_ + "/" + confinement_type();
+    !confined() && !boost::filesystem::exists(dir, ec) && ::mkdir(dir.c_str(), 0700);
+
+    // A confined scope is allowed to create this dir.
+    dir += "/" + demangled_id(scope_id_);
     !boost::filesystem::exists(dir, ec) && ::mkdir(dir.c_str(), 0700);
-    dir += "/" + id;
+
+    return dir;
+}
+
+string RuntimeImpl::find_app_dir() const
+{
+    // Create the app_dir_/<id> directories if they don't exist.
+    boost::system::error_code ec;
+    !confined() && !boost::filesystem::exists(app_dir_, ec) && ::mkdir(app_dir_.c_str(), 0700);
+    string dir = app_dir_ + "/" + demangled_id(scope_id_);
+    !confined() && !boost::filesystem::exists(dir, ec) && ::mkdir(dir.c_str(), 0700);
+
+    return dir;
+}
+
+string RuntimeImpl::find_log_dir(string const& id) const
+{
+    // Create the log_dir_/<confinement-type>/<id>/logs directories if they don't exist.
+    boost::system::error_code ec;
+    !confined() && !boost::filesystem::exists(log_dir_, ec) && ::mkdir(log_dir_.c_str(), 0700);
+    string dir = log_dir_ + "/" + confinement_type();
+    !confined() && !boost::filesystem::exists(dir, ec) && ::mkdir(dir.c_str(), 0700);
+
+    // A confined scope is allowed to create this dir.
+    dir += "/" + demangled_id(id);
     !boost::filesystem::exists(dir, ec) && ::mkdir(dir.c_str(), 0700);
+    dir += "/logs";
+    !boost::filesystem::exists(dir, ec) && ::mkdir(dir.c_str(), 0700);
+
+    return dir;
+}
+
+string RuntimeImpl::find_tmp_dir() const
+{
+    // Set tmp dir.
+    // We need to create any directories under /run/user/<uid> because they might not
+    // exist. We set the sticky bit because, without this, things in
+    // /run/user may be deleted if not accessed for more than six hours.
+    string dir = string("/run/user/") + std::to_string(geteuid());
+    dir += "/scopes";
+    boost::system::error_code ec;
+    !confined() && !boost::filesystem::exists(dir, ec) && ::mkdir(dir.c_str(), 0700 | S_ISVTX);
+    dir += "/" + confinement_type();
+    !confined() && !boost::filesystem::exists(dir, ec) && ::mkdir(dir.c_str(), 0700 | S_ISVTX);
+
+    // A confined scope is allowed to create this dir.
+    dir += "/" + scope_id_;  // Not demangled, use the real scope ID.
+    !boost::filesystem::exists(dir, ec) && ::mkdir(dir.c_str(), 0700 | S_ISVTX);
 
     return dir;
 }
