@@ -24,7 +24,6 @@
 #include <unity/scopes/internal/Utils.h>
 
 #include <algorithm>
-#include <iostream>
 #include <sstream>
 #include <string>
 
@@ -56,6 +55,10 @@ namespace
     const string idle_timeout_key = "IdleTimeout";
     const string results_ttl_key = "ResultsTtlType";
     const string debug_mode_key = "DebugMode";
+    const string child_scope_ids_key = "ChildScopes";
+    const string version_key = "Version";
+    const string keywords_key = "Keywords";
+    const string is_aggregator_key = "IsAggregator";
 
     const string scope_appearance_group = "Appearance";
     const string fg_color_key = "ForegroundColor";
@@ -190,6 +193,61 @@ ScopeConfig::ScopeConfig(string const& configfile) :
     {
     }
 
+    try
+    {
+        child_scope_ids_ = parser()->get_string_array(scope_config_group, child_scope_ids_key);
+    }
+    catch (LogicException const&)
+    {
+    }
+
+    for (auto const& id: child_scope_ids_)
+    {
+        if (id.size() == 0)
+        {
+            throw_ex("Invalid empty scope id for ChildScopes");
+        }
+    }
+
+    try
+    {
+        version_ = get_int(scope_config_group, version_key);
+        if (version_ <= 0)
+        {
+            throw_ex("Version must be > 0");
+        }
+    }
+    catch (LogicException const&)
+    {
+        version_ = 0;
+    }
+
+    try
+    {
+        auto keywords_vec = parser()->get_string_array(scope_config_group, keywords_key);
+        keywords_ = set<string>(keywords_vec.begin(), keywords_vec.end());
+    }
+    catch (LogicException const&)
+    {
+    }
+
+    for (auto const& keyword: keywords_)
+    {
+        if (keyword.empty())
+        {
+            throw_ex("Invalid empty keyword string found in \"Keywords\" list");
+        }
+    }
+
+    try
+    {
+        is_aggregator_ = parser()->get_boolean(scope_config_group, is_aggregator_key);
+    }
+    catch (LogicException const&)
+    {
+        is_aggregator_ = false;
+    }
+
     try
     {
         debug_mode_ = parser()->get_boolean(scope_config_group, debug_mode_key);
@@ -227,7 +285,11 @@ ScopeConfig::ScopeConfig(string const& configfile) :
                scoperunner_key,
                idle_timeout_key,
                results_ttl_key,
-               debug_mode_key
+               debug_mode_key,
+               child_scope_ids_key,
+               version_key,
+               keywords_key,
+               is_aggregator_key
            }
         },
         {  scope_appearance_group,
@@ -401,6 +463,26 @@ VariantMap ScopeConfig::appearance_attributes() const
     return appearance_attributes_;
 }
 
+std::vector<std::string> ScopeConfig::child_scope_ids() const
+{
+    return child_scope_ids_;
+}
+
+int ScopeConfig::version() const
+{
+    return version_;
+}
+
+std::set<std::string> ScopeConfig::keywords() const
+{
+    return keywords_;
+}
+
+bool ScopeConfig::is_aggregator() const
+{
+    return is_aggregator_;
+}
+
 } // namespace internal
 
 } // namespace scopes
