@@ -27,11 +27,12 @@
 #include <unity/scopes/internal/QueryObject.h>
 #include <unity/scopes/internal/RuntimeImpl.h>
 #include <unity/scopes/internal/ScopeBaseImpl.h>
+#include <unity/scopes/internal/SearchMetadataImpl.h>
+#include <unity/scopes/internal/SearchQueryBaseImpl.h>
 #include <unity/scopes/ScopeBase.h>
 #include <unity/UnityExceptions.h>
 
 #include <cassert>
-#include <iostream> // TODO: remove this once logging is added
 #include <sstream>
 
 using namespace std;
@@ -47,12 +48,10 @@ namespace scopes
 namespace internal
 {
 
-ScopeObject::ScopeObject(RuntimeImpl* runtime, ScopeBase* scope_base, bool debug_mode) :
-    runtime_(runtime),
+ScopeObject::ScopeObject(ScopeBase* scope_base, bool debug_mode) :
     scope_base_(scope_base),
     debug_mode_(debug_mode)
 {
-    assert(runtime);
     assert(scope_base);
 }
 
@@ -60,7 +59,9 @@ ScopeObject::~ScopeObject()
 {
 }
 
-MWQueryCtrlProxy ScopeObject::query(MWReplyProxy const& reply, MiddlewareBase* mw_base,
+MWQueryCtrlProxy ScopeObject::query(MWReplyProxy const& reply,
+        MiddlewareBase* mw_base,
+        std::string const& method,
         std::function<QueryBase::SPtr()> const& query_factory_fun,
         std::function<QueryObjectBase::SPtr(QueryBase::SPtr, MWQueryCtrlProxy)> const& query_object_factory_fun)
 {
@@ -71,7 +72,8 @@ MWQueryCtrlProxy ScopeObject::query(MWReplyProxy const& reply, MiddlewareBase* m
         // to be safe, we don't assert, in case someone is running a broken client.
 
         // TODO: log error about incoming request containing an invalid reply proxy.
-        throw LogicException("Scope \"" + runtime_->scope_id() + "\": query() called with null reply proxy");
+        throw LogicException("Scope \"" + mw_base->runtime()->scope_id() + "\": "
+                             + method + " called with null reply proxy");
     }
 
     // Ask scope to instantiate a new query.
@@ -81,16 +83,16 @@ MWQueryCtrlProxy ScopeObject::query(MWReplyProxy const& reply, MiddlewareBase* m
         query_base = query_factory_fun();
         if (!query_base)
         {
-            string msg = "Scope \"" + runtime_->scope_id() + "\" returned nullptr from query_factory_fun()";
-            cerr << msg << endl;
+            string msg = "Scope \"" + mw_base->runtime()->scope_id() + "\" returned nullptr from " + method + "()";
+            BOOST_LOG(mw_base->runtime()->logger()) << msg;
             throw ResourceException(msg);
         }
         query_base->p->set_settings_db(scope_base_->p->settings_db());
     }
     catch (...)
     {
-        string msg = "Scope \"" + runtime_->scope_id() + "\" threw an exception from query_factory_fun()";
-        cerr << msg << endl;
+        string msg = "Scope \"" + mw_base->runtime()->scope_id() + "\" threw an exception from " + method + "()";
+        BOOST_LOG(mw_base->runtime()->logger()) << msg;
         throw ResourceException(msg);
     }
 
@@ -128,8 +130,7 @@ MWQueryCtrlProxy ScopeObject::query(MWReplyProxy const& reply, MiddlewareBase* m
         catch (...)
         {
         }
-        cerr << "query(): " << e.what() << endl;
-        // TODO: log error
+        BOOST_LOG(mw_base->runtime()->logger()) << "ScopeObject::query(): " << e.what();
         throw;
     }
     catch (...)
@@ -141,8 +142,7 @@ MWQueryCtrlProxy ScopeObject::query(MWReplyProxy const& reply, MiddlewareBase* m
         catch (...)
         {
         }
-        cerr << "query(): unknown exception" << endl;
-        // TODO: log error
+        BOOST_LOG(mw_base->runtime()->logger()) << "ScopeObject::query(): unknown exception";
         throw;
     }
     return ctrl_proxy;
@@ -150,35 +150,70 @@ MWQueryCtrlProxy ScopeObject::query(MWReplyProxy const& reply, MiddlewareBase* m
 
 MWQueryCtrlProxy ScopeObject::search(CannedQuery const& q,
                                      SearchMetadata const& hints,
+                                     VariantMap const& context,
                                      MWReplyProxy const& reply,
                                      InvokeInfo const& info)
 {
-    return query(reply, info.mw,
-            [&q, &hints, this]() -> SearchQueryBase::UPtr {
-                 auto search_query = this->scope_base_->search(q, hints);
-                 search_query->set_department_id(q.department_id());
-                 return search_query;
-            },
-            [&reply, &hints](QueryBase::SPtr query_base, MWQueryCtrlProxy ctrl_proxy) -> QueryObjectBase::SPtr {
-                return make_shared<QueryObject>(query_base, hints.cardinality(), reply, ctrl_proxy);
-            }
+    return query(reply,
+                 info.mw,
+                 "search",
+                 [&q, &hints, &context, this]() -> SearchQueryBase::UPtr {
+                      auto search_query = this->scope_base_->search(q, hints);
+                      search_query->set_department_id(q.department_id());
+
+                      auto sqb = dynamic_cast<SearchQueryBaseImpl*>(search_query->fwd());
+                      assert(sqb);
+
+                      // Set client ID and history that we received in the SearchQueryBase
+                      // for loop detection.
+                      auto const c_it = context.find("client_id");
+                      if (c_it != context.end())
+                      {
+                          string client_id;
+                          client_id = c_it->second.get_string();
+                          sqb->set_client_id(client_id);
+                      }
+
+                      auto const h_it = context.find("history");
+                      if (h_it != context.end())
+                      {
+                         auto const hlist = h_it->second.get_array();
+                         SearchQueryBaseImpl::History history;
+                         for (auto const& t : hlist)
+                         {
+                             string client_id = t.get_dict()["c"].get_string();
+                             string agg = t.get_dict()["a"].get_string();
+                             string recv = t.get_dict()["r"].get_string();
+                             SearchQueryBaseImpl::HistoryData hd = make_tuple(client_id, agg, recv);
+                             history.push_back(hd);
+                         }
+                         sqb->set_history(history);
+                      }
+
+                      return search_query;
+                 },
+                 [&reply, &hints, this](QueryBase::SPtr query_base, MWQueryCtrlProxy ctrl_proxy) -> QueryObjectBase::SPtr {
+                     return make_shared<QueryObject>(query_base, hints.cardinality(), reply, ctrl_proxy);
+                 }
     );
 }
 
 MWQueryCtrlProxy ScopeObject::activate(Result const& result,
-                                           ActionMetadata const& hints,
-                                           MWReplyProxy const& reply,
-                                           InvokeInfo const& info)
+                                       ActionMetadata const& hints,
+                                       MWReplyProxy const& reply,
+                                       InvokeInfo const& info)
 {
-    return query(reply, info.mw,
-            [&result, &hints, this]() -> QueryBase::SPtr {
-                return this->scope_base_->activate(result, hints);
-            },
-            [&reply](QueryBase::SPtr query_base, MWQueryCtrlProxy ctrl_proxy) -> QueryObjectBase::SPtr {
-                auto activation_base = dynamic_pointer_cast<ActivationQueryBase>(query_base);
-                assert(activation_base);
-                return make_shared<ActivationQueryObject>(activation_base, reply, ctrl_proxy);
-            }
+    return query(reply,
+                 info.mw,
+                 "activate",
+                 [&result, &hints, this]() -> QueryBase::SPtr {
+                     return this->scope_base_->activate(result, hints);
+                 },
+                 [&reply, this](QueryBase::SPtr query_base, MWQueryCtrlProxy ctrl_proxy) -> QueryObjectBase::SPtr {
+                     auto activation_base = dynamic_pointer_cast<ActivationQueryBase>(query_base);
+                     assert(activation_base);
+                     return make_shared<ActivationQueryObject>(activation_base, reply, ctrl_proxy);
+                 }
     );
 }
 
@@ -189,15 +224,17 @@ MWQueryCtrlProxy ScopeObject::perform_action(Result const& result,
                                              MWReplyProxy const &reply,
                                              InvokeInfo const& info)
 {
-    return query(reply, info.mw,
-            [&result, &hints, &widget_id, &action_id, this]() -> QueryBase::SPtr {
-                return this->scope_base_->perform_action(result, hints, widget_id, action_id);
-            },
-            [&reply](QueryBase::SPtr query_base, MWQueryCtrlProxy ctrl_proxy) -> QueryObjectBase::SPtr {
-                auto activation_base = dynamic_pointer_cast<ActivationQueryBase>(query_base);
-                assert(activation_base);
-                return make_shared<ActivationQueryObject>(activation_base, reply, ctrl_proxy);
-            }
+    return query(reply,
+                 info.mw,
+                 "perform_action",
+                 [&result, &hints, &widget_id, &action_id, this]() -> QueryBase::SPtr {
+                     return this->scope_base_->perform_action(result, hints, widget_id, action_id);
+                 },
+                 [&reply, this](QueryBase::SPtr query_base, MWQueryCtrlProxy ctrl_proxy) -> QueryObjectBase::SPtr {
+                     auto activation_base = dynamic_pointer_cast<ActivationQueryBase>(query_base);
+                     assert(activation_base);
+                     return make_shared<ActivationQueryObject>(activation_base, reply, ctrl_proxy);
+                 }
     );
 }
 
@@ -206,18 +243,30 @@ MWQueryCtrlProxy ScopeObject::preview(Result const& result,
                                       MWReplyProxy const& reply,
                                       InvokeInfo const& info)
 {
-    return query(reply, info.mw,
-            [&result, &hints, this]() -> QueryBase::SPtr {
-                return this->scope_base_->preview(result, hints);
-            },
-            [&reply](QueryBase::SPtr query_base, MWQueryCtrlProxy ctrl_proxy) -> QueryObjectBase::SPtr {
-                auto preview_query = dynamic_pointer_cast<PreviewQueryBase>(query_base);
-                assert(preview_query);
-                return make_shared<PreviewQueryObject>(preview_query, reply, ctrl_proxy);
-            }
+    return query(reply,
+                 info.mw,
+                 "preview",
+                 [&result, &hints, this]() -> QueryBase::SPtr {
+                     return this->scope_base_->preview(result, hints);
+                 },
+                 [&reply, this](QueryBase::SPtr query_base, MWQueryCtrlProxy ctrl_proxy) -> QueryObjectBase::SPtr {
+                     auto preview_query = dynamic_pointer_cast<PreviewQueryBase>(query_base);
+                     assert(preview_query);
+                     return make_shared<PreviewQueryObject>(preview_query, reply, ctrl_proxy);
+                 }
     );
 }
 
+ChildScopeList ScopeObject::child_scopes() const
+{
+    return scope_base_->child_scopes();
+}
+
+bool ScopeObject::set_child_scopes(ChildScopeList const& child_scopes)
+{
+    return scope_base_->p->set_child_scopes(child_scopes);
+}
+
 bool ScopeObject::debug_mode() const
 {
     return debug_mode_;
