@@ -16,12 +16,13 @@
  * Authored by: Marcus Tomlinson <marcus.tomlinson@canonical.com>
  */
 
-#include <unity/scopes/internal/QueryBaseImpl.h>
 #include <unity/scopes/internal/smartscopes/SSScopeObject.h>
-#include <unity/scopes/internal/smartscopes/SmartScope.h>
-#include <unity/scopes/internal/smartscopes/SSQueryObject.h>
 #include <unity/scopes/internal/MWQuery.h>
 #include <unity/scopes/internal/MWReply.h>
+#include <unity/scopes/internal/QueryBaseImpl.h>
+#include <unity/scopes/internal/RuntimeImpl.h>
+#include <unity/scopes/internal/smartscopes/SmartScope.h>
+#include <unity/scopes/internal/smartscopes/SSQueryObject.h>
 #include <unity/scopes/ScopeExceptions.h>
 #include <unity/scopes/ActionMetadata.h>
 #include <unity/scopes/SearchMetadata.h>
@@ -63,9 +64,10 @@ SSScopeObject::~SSScopeObject()
 }
 
 MWQueryCtrlProxy SSScopeObject::search(CannedQuery const& q,
-                                             SearchMetadata const& hints,
-                                             MWReplyProxy const& reply,
-                                             InvokeInfo const& info)
+                                       SearchMetadata const& hints,
+                                       VariantMap const& /* context */,
+                                       MWReplyProxy const& reply,
+                                       InvokeInfo const& info)
 {
     return query(info,
                  reply,
@@ -171,8 +173,7 @@ MWQueryCtrlProxy SSScopeObject::query(InvokeInfo const& info,
         catch (...)
         {
         }
-        std::cerr << "query(): " << e.what() << std::endl;
-        // TODO: log error
+        BOOST_LOG(info.mw->runtime()->logger()) << "SSScopeObject::query(): " << e.what();
         throw;
     }
     catch (...)
@@ -184,14 +185,23 @@ MWQueryCtrlProxy SSScopeObject::query(InvokeInfo const& info,
         catch (...)
         {
         }
-        std::cerr << "query(): unknown exception" << std::endl;
-        // TODO: log error
+        BOOST_LOG(info.mw->runtime()->logger()) << "SSScopeObject::query(): unknown exception";
         throw;
     }
 
     return info.mw->create_query_ctrl_proxy(reply->identity() + ".c", info.mw->get_query_ctrl_endpoint());
 }
 
+ChildScopeList SSScopeObject::child_scopes() const
+{
+    return ChildScopeList();
+}
+
+bool SSScopeObject::set_child_scopes(ChildScopeList const&)
+{
+    return false;
+}
+
 bool SSScopeObject::debug_mode() const
 {
     return false;
