@@ -19,17 +19,18 @@
 #include <unity/scopes/internal/zmq_middleware/ScopeI.h>
 
 #include <scopes/internal/zmq_middleware/capnproto/Scope.capnp.h>
-#include <unity/scopes/internal/ResultImpl.h>
-#include <unity/scopes/internal/CannedQueryImpl.h>
-#include <unity/scopes/internal/ScopeObject.h>
+#include <unity/scopes/CannedQuery.h>
 #include <unity/scopes/internal/ActionMetadataImpl.h>
+#include <unity/scopes/internal/CannedQueryImpl.h>
+#include <unity/scopes/internal/ResultImpl.h>
 #include <unity/scopes/internal/SearchMetadataImpl.h>
+#include <unity/scopes/internal/ScopeMetadataImpl.h>
+#include <unity/scopes/internal/ScopeObject.h>
 #include <unity/scopes/internal/zmq_middleware/ObjectAdapter.h>
 #include <unity/scopes/internal/zmq_middleware/VariantConverter.h>
 #include <unity/scopes/internal/zmq_middleware/ZmqQueryCtrl.h>
 #include <unity/scopes/internal/zmq_middleware/ZmqReply.h>
 #include <unity/scopes/internal/zmq_middleware/ZmqScope.h>
-#include <unity/scopes/CannedQuery.h>
 
 #include <cassert>
 
@@ -60,20 +61,24 @@ interface Scope
     QueryCtrl* preview(ValueDict result, ValueDict hints, Reply* replyProxy);
     QueryCtrl* perform_action(ValueDict result, ValueDict hints, string action_id, Reply* replyProxy);
     QueryCtrl* activate(ValueDict result, ValueDict hints, Reply* replyProxy);
+    ChildScopeList child_scopes();
+    bool set_child_scopes(ChildScopeList const& child_scopes);
     bool debug_mode();
 };
 
 */
 
-using namespace std::placeholders;
+using namespace std;
+namespace ph = std::placeholders;
 
 ScopeI::ScopeI(ScopeObjectBase::SPtr const& so) :
-    ServantBase(so, {
-        { "search", bind(&ScopeI::search_, this, _1, _2, _3) },
-        { "preview", bind(&ScopeI::preview_, this, _1, _2, _3) },
-        { "activate", bind(&ScopeI::activate_, this, _1, _2, _3) },
-        { "perform_action", bind(&ScopeI::perform_action_, this, _1, _2, _3) },
-        { "debug_mode", bind(&ScopeI::debug_mode_, this, _1, _2, _3) }
+    ServantBase(so, { { "search", bind(&ScopeI::search_, this, ph::_1, ph::_2, ph::_3) },
+                      { "preview", bind(&ScopeI::preview_, this, ph::_1, ph::_2, ph::_3) },
+                      { "activate", bind(&ScopeI::activate_, this, ph::_1, ph::_2, ph::_3) },
+                      { "perform_action", bind(&ScopeI::perform_action_, this, ph::_1, ph::_2, ph::_3) },
+                      { "child_scopes", bind(&ScopeI::child_scopes_, this, ph::_1, ph::_2, ph::_3) },
+                      { "set_child_scopes", bind(&ScopeI::set_child_scopes_, this, ph::_1, ph::_2, ph::_3) },
+                      { "debug_mode", bind(&ScopeI::debug_mode_, this, ph::_1, ph::_2, ph::_3) }
     })
 {
 }
@@ -94,10 +99,12 @@ void ScopeI::search_(Current const& current,
                               proxy.getEndpoint().cStr(),
                               proxy.getIdentity().cStr(),
                               proxy.getCategory().cStr()));
+    auto context = to_variant_map(req.getContext());
     auto delegate = dynamic_pointer_cast<ScopeObjectBase>(del());
     assert(delegate);
     auto ctrl_proxy = dynamic_pointer_cast<ZmqQueryCtrl>(delegate->search(query,
                                                                           metadata,
+                                                                          context,
                                                                           reply_proxy,
                                                                           to_info(current)));
     assert(ctrl_proxy);
@@ -194,6 +201,74 @@ void ScopeI::preview_(Current const& current,
     p.setCategory(ctrl_proxy->target_category().c_str());
 }
 
+void ScopeI::child_scopes_(Current const&,
+                           capnp::AnyPointer::Reader&,
+                           capnproto::Response::Builder& r)
+{
+    auto delegate = dynamic_pointer_cast<ScopeObjectBase>(del());
+    assert(delegate);
+
+    auto child_scopes = delegate->child_scopes();
+
+    r.setStatus(capnproto::ResponseStatus::SUCCESS);
+    auto list_response = r.initPayload().getAs<capnproto::Scope::ChildScopesResponse>();
+    auto list = list_response.initReturnValue(child_scopes.size());
+
+    for (size_t i = 0; i < child_scopes.size(); ++i)
+    {
+        list[i].setId(child_scopes[i].id);
+
+        auto dict = list[i].initMetadata();
+        to_value_dict(child_scopes[i].metadata.serialize(), dict);
+
+        list[i].setEnabled(child_scopes[i].enabled);
+
+        auto keywords = list[i].initKeywords(child_scopes[i].keywords.size());
+        int j = 0;
+        for (auto const& kw : child_scopes[i].keywords)
+        {
+            keywords.set(j++, kw);
+        }
+    }
+}
+
+void ScopeI::set_child_scopes_(Current const& current,
+                               capnp::AnyPointer::Reader& in_params,
+                               capnproto::Response::Builder& r)
+{
+    auto delegate = std::dynamic_pointer_cast<ScopeObjectBase>(del());
+    assert(delegate);
+
+    auto list = in_params.getAs<capnproto::Scope::SetChildScopesRequest>().getChildScopes();
+
+    ChildScopeList child_scope_list;
+    for (size_t i = 0; i < list.size(); ++i)
+    {
+        string id = list[i].getId();
+
+        auto md = list[i].getMetadata();
+        VariantMap m = to_variant_map(md);
+        unique_ptr<ScopeMetadataImpl> smdi(new ScopeMetadataImpl(m, current.adapter->mw()));
+        auto metadata = ScopeMetadata(ScopeMetadataImpl::create(move(smdi)));
+
+        bool enabled = list[i].getEnabled();
+
+        set<string> keywords;
+        auto keywords_capn = list[i].getKeywords();
+        for (auto const& kw : keywords_capn)
+        {
+            keywords.emplace(kw);
+        }
+
+        child_scope_list.push_back( ChildScope{id, metadata, enabled, keywords} );
+    }
+
+    bool result = delegate->set_child_scopes(child_scope_list);
+    r.setStatus(capnproto::ResponseStatus::SUCCESS);
+    auto response = r.initPayload().getAs<capnproto::Scope::SetChildScopesResponse>();
+    response.setReturnValue(result);
+}
+
 void ScopeI::debug_mode_(Current const&,
                          capnp::AnyPointer::Reader&,
                          capnproto::Response::Builder& r)
