@@ -16,12 +16,16 @@
  * Authored by: Marcus Tomlinson <marcus.tomlinson@canonical.com>
  */
 
-#include <unity/scopes/OptionSelectorFilter.h>
-#include <unity/scopes/internal/smartscopes/HttpClientQt.h>
 #include <unity/scopes/internal/JsonCppNode.h>
+#include <unity/scopes/internal/Logger.h>
+#include <unity/scopes/internal/smartscopes/HttpClientNetCpp.h>
 #include <unity/scopes/internal/smartscopes/SmartScopesClient.h>
+#include <unity/scopes/OptionSelectorFilter.h>
 
 #include <unity/UnityExceptions.h>
+#include <unity/util/FileIO.h>
+#include <boost/filesystem/operations.hpp>
+#include <fstream>
 
 #include "../RaiiServer.h"
 
@@ -42,12 +46,28 @@ class SmartScopesClientTest : public Test
 {
 public:
     SmartScopesClientTest()
-        : http_client_(new HttpClientQt(20000)),
+        : http_client_(new HttpClientNetCpp(20000)),
           json_node_(new JsonCppNode()),
-          server_(FAKE_SSS_PATH)
+          server_(FAKE_SSS_PATH, FAKE_SSS_LOG)
     {
+        boost::filesystem::remove(FAKE_SSS_LOG);
         sss_url_ = "http://127.0.0.1:" + std::to_string(server_.port_);
-        ssc_ = std::make_shared<SmartScopesClient>(http_client_, json_node_, sss_url_);
+        ssc_ = std::make_shared<SmartScopesClient>(http_client_, json_node_, nullptr, sss_url_, PARTNER_FILE);
+    }
+
+    bool grep_string(std::string const &s)
+    {
+        std::stringstream str(unity::util::read_text_file(FAKE_SSS_LOG));
+        while (str)
+        {
+            char tmp[1024];
+            str.getline(tmp, 1024);
+            if (strstr(tmp, s.c_str()))
+            {
+                return true;
+            }
+        }
+        return false;
     }
 
 protected:
@@ -82,8 +102,10 @@ TEST_F(SmartScopesClientTest, remote_scopes)
     EXPECT_EQ("icon", *scopes[0].icon);
     EXPECT_EQ(nullptr, scopes[0].art);
     EXPECT_FALSE(scopes[0].invisible);
+    EXPECT_EQ(0, scopes[0].version);
     EXPECT_EQ(nullptr, scopes[0].appearance);
     EXPECT_EQ(nullptr, scopes[0].settings);
+    EXPECT_TRUE(scopes[0].keywords.empty());
 
     EXPECT_EQ("dummy.scope.2", scopes[1].id);
     EXPECT_EQ("Dummy Demo Scope 2", scopes[1].name);
@@ -93,9 +115,11 @@ TEST_F(SmartScopesClientTest, remote_scopes)
     EXPECT_EQ(nullptr, scopes[1].icon);
     EXPECT_EQ("art", *scopes[1].art);
     EXPECT_TRUE(scopes[1].invisible);
+    EXPECT_EQ(2, scopes[1].version);
     EXPECT_EQ("#00BEEF", (*scopes[1].appearance)["background"].get_string());
     EXPECT_EQ("logo.png", (*scopes[1].appearance)["PageHeader"].get_dict()["logo"].get_string());
     EXPECT_EQ(nullptr, scopes[1].settings);
+    EXPECT_TRUE(scopes[1].keywords.empty());
 
     EXPECT_EQ("dummy.scope.3", scopes[2].id);
     EXPECT_EQ("Dummy Demo Scope 3", scopes[2].name);
@@ -113,6 +137,21 @@ TEST_F(SmartScopesClientTest, remote_scopes)
               "\"defaultValue\":23},\"type\":\"number\"},{\"displayName\":\"Enabled\",\"id\":"
               "\"enabled\",\"parameters\":{\"defaultValue\":true},\"type\":\"boolean\"}]\n",
               *scopes[2].settings);
+    ASSERT_EQ(4, scopes[2].keywords.size());
+    EXPECT_NE(scopes[2].keywords.end(), scopes[2].keywords.find("music"));
+    EXPECT_NE(scopes[2].keywords.end(), scopes[2].keywords.find("video"));
+    EXPECT_NE(scopes[2].keywords.end(), scopes[2].keywords.find("news"));
+    EXPECT_NE(scopes[2].keywords.end(), scopes[2].keywords.find("games"));
+
+    EXPECT_TRUE(grep_string("/remote-scopes : partner=Partner%20String"));
+}
+
+TEST_F(SmartScopesClientTest, remote_scopes_no_partner)
+{
+    std::vector<RemoteScope> scopes;
+    auto ssc_no_partner_ = std::make_shared<SmartScopesClient>(http_client_, json_node_, nullptr, sss_url_, "/this/file/doesnt/exist");
+    EXPECT_TRUE(ssc_no_partner_->get_remote_scopes(scopes, "", false));
+    EXPECT_FALSE(grep_string("/remote-scopes : partner"));
 }
 
 TEST_F(SmartScopesClientTest, search)
@@ -140,7 +179,7 @@ TEST_F(SmartScopesClientTest, search)
         dept = deptinfo;
     };
 
-    auto search_handle = ssc_->search(handler, sss_url_ + "/demo", "stuff", "", "session_id", 0, "platform", VariantMap(), VariantMap(), "en_US", "", "ThisIsUserAgentHeader");
+    auto search_handle = ssc_->search(handler, sss_url_ + "/demo", "stuff", "", "session_id", 0, "platform", VariantMap(), VariantMap(), "en_US", LocationInfo(), "ThisIsUserAgentHeader");
     search_handle->wait();
 
     ASSERT_EQ(3u, results.size());
@@ -238,7 +277,7 @@ TEST_F(SmartScopesClientTest, userAgentHeader)
     handler.departments_handler = [](std::shared_ptr<DepartmentInfo> const&) {
     };
 
-    auto search_handle = ssc_->search(handler, sss_url_ + "/demo", "test_user_agent_header", "", "session_id", 0, "platform", VariantMap(), VariantMap(), "en_US", "", "ThisIsUserAgentHeader");
+    auto search_handle = ssc_->search(handler, sss_url_ + "/demo", "test_user_agent_header", "", "session_id", 0, "platform", VariantMap(), VariantMap(), "en_US", LocationInfo(), "ThisIsUserAgentHeader");
     search_handle->wait();
 
     ASSERT_EQ(4u, results.size());
