@@ -421,6 +421,7 @@ public:
 
     virtual MWQueryCtrlProxy search(CannedQuery const&,
                                           SearchMetadata const&,
+                                          VariantMap const&,
                                           MWReplyProxy const&,
                                           InvokeInfo const&) override
     {
@@ -430,7 +431,7 @@ public:
     virtual MWQueryCtrlProxy activate(Result const&,
                                       ActionMetadata const&,
                                       MWReplyProxy const&,
-                                      InvokeInfo const&)
+                                      InvokeInfo const&) override
     {
         return nullptr;
     }
@@ -453,6 +454,16 @@ public:
         return nullptr;
     }
 
+    virtual ChildScopeList child_scopes() const override
+    {
+        return ChildScopeList();
+    }
+
+    virtual bool set_child_scopes(ChildScopeList const&) override
+    {
+        return false;
+    }
+
     virtual bool debug_mode() const override
     {
         return false;
